/*--------------------------------------------------------------------
 * $Id: uvspec.h,v 1.73 2003/11/27 17:48:13 pa2h Exp $
 * 
 * This file is part of libRadtran.
 * Copyright (c) 1997-2001 by Arve Kylling and Bernhard Mayer.
 * 
 * ######### Contact info: http://www.libradtran.org #########
 *
 * This program is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License   
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.        
 * 
 * This program is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of  
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the   
 * GNU General Public License for more details.                    
 * 
 * You should have received a copy of the GNU General Public License          
 * along with this program; if not, write to the Free Software                
 * Foundation, Inc., 59 Temple Place - Suite 330, 
 * Boston, MA 02111-1307, USA.
 *--------------------------------------------------------------------*/

#ifndef __uvspec_h
#define __uvspec_h

#include <stdio.h>
#include <math.h>
#include <string.h>
#include <stdlib.h>

#define VERSION "1.0-beta"

/* Indices for the reading of the atmosphere files */
#define ATMFILE_ZD       0
#define ATMFILE_PRESS    1
#define ATMFILE_TEMPER   2
#define ATMFILE_AIR      3
#define ATMFILE_O3       4
#define ATMFILE_O2       5
#define ATMFILE_H2O      6
#define ATMFILE_CO2      7
#define ATMFILE_NO2      8


/* Atmospherid molecules that may be included in spectral calculations */
#define MOL_NN        9
#define MOL_AIR       0
#define MOL_O3        1
#define MOL_O2        2
#define MOL_H2O       3
#define MOL_CO2       4
#define MOL_NO2       5
#define MOL_BRO       6
#define MOL_OCLO      7
#define MOL_HCHO      8

/* Possible units of total molecule column */
#define CM_2          1
#define DU            2


#define FN_NN               25  /* Number of general files, increase 
				   when adding new general files */
#define FN_PATH              0
#define FN_ALBEDO            1
#define FN_ATMOSPHERE        2
#define FN_EXTRATERRESTRIAL  3
#define FN_SZA               4
#define FN_WLTRANS           5
#define FN_WLBANDS           6 
#define FN_WCLOUD            7
#define FN_WCLOUD_FILES      8 
#define FN_WCLOUD_PROP       9
#define FN_WCLOUD_IPA       10
#define FN_WCLOUD_IPA_3D    11
#define FN_ICLOUD           12
#define FN_ICLOUD_FILES     13 
#define FN_ICLOUD_PROP      14 
#define FN_ICLOUD_IPA       15 
#define FN_SLITFUNCTION     16
#define FN_FILTERFUNCTION   17
#define FN_SPLINE           18
#define FN_RAYLEIGH         19
#define FN_MOLABS           20
#define FN_RPV              21
#define FN_REFIND           22
#define FN_CK_GENERIC       23
#define FN_RH               24

#define FN_AER_NN            7  /* Number of aerosol related files, increase 
				   when adding new aerosol files */
#define FN_AER_TAU           0
#define FN_AER_GG            1
#define FN_AER_SSA           2
#define FN_AER_MOM           3
#define FN_AER_REF           4
#define FN_AER_SIZ           5
#define FN_AER_FILES         6

#define FN_MC_NN            10  /* Number of mystic related files, increase 
				   when adding new mystic files */
#define FN_MC_PHOTONS        0
#define FN_MC_MOMCLD         1
#define FN_MC_MOMTRNC        2
#define FN_MC_PCLD           3
#define FN_MC_BASENAME       4
#define FN_MC_ALBEDO         5
#define FN_MC_ELEVATION      6
#define FN_MC_RPV            7
#define FN_MC_RPVTYPE        8
#define FN_MC_WCLOUD         9


#define CK_NONE         0
#define CK_KATO         1
#define CK_KATO2        2
#define CK_KATO2_96     3
#define CK_FU           4
#define CK_AVHRR_KRATZ  5
#define CK_GENERIC      6
#define CK_SBDART       7

#define WLGRID_NONE    0
#define WLGRID_CK      1
#define WLGRID_USER    2
#define WLGRID_BANDS   3
#define WLGRID_MOLABS  4
#define WLGRID_UVSPEC  5

#define SRC_NONE    0
#define SRC_SOLAR   1
#define SRC_THERMAL 2

#define O3_MOLINA    1
#define O3_DAUMONT   2
#define O3_BASSPAUR  3    

#define NO2_GOME     1

#define BRO_IASB     1

#define OCLO_WAHNER  1

#define HCHO_IASB  1

#define RAYLEIGH_NONE 0
#define RAYLEIGH_CALC 1
#define RAYLEIGH_FILE 2

#define MOLABS_NONE      0
#define MOLABS_CALC      1
#define MOLABS_FILE_MONO 2
#define MOLABS_FILE_SPEC 3

#define MX_NN  9  
#define MX_O2  0
#define MX_CO2 1
#define MX_H2O 2
#define MX_CH4 3
#define MX_N2O 4
#define MX_NO2 5
#define MX_F11 6
#define MX_F12 7
#define MX_F22 8

#define CLD_ZD     0    
#define CLD_LWC    1
#define CLD_EFFR   2

#define BRDF_NONE   0
#define BRDF_RPV    1  /* don't change these numbers as they are used by Fortran code */
#define BRDF_CM     2  /* which of course has no access to this header file           */
#define BRDF_RPV2D  3
#define BRDF_CM2D   4

#define DISORT      1
#define SDISORT     2
#define TWOSTR      3
#define SUCORD      4
#define MONTECARLO  5
#define POLRADTRAN  6
#define DISORT2     7
#define SPSDISORT   8
#define QDISORT     9
#define TZS        10
#define SSS        11
#define SSSI       12

#define MOMCLD_NONE       0
#define MOMCLD_WATER      1
#define MOMCLD_ICE        2
#define MOMCLD_FILE       3

#define WCPROP_NONE       0   /* no water cloud                           */
#define WCPROP_HU         1   /* "wc_file" and Hu and Stamnes [1993]      */
#define WCPROP_MIE        2   /* "wc_file" and "wc_properties mie"        */
#define WCPROP_FILE       3   /* "wc_file" and "wc_properties <filename>" */
#define WCPROP_EXPLICIT   4   /* "wc_files"                               */

#define ICPROP_NONE       0   /* no ice cloud                             */
#define ICPROP_FU         1   /* "ic_file" and Fu et al. [1996/98]        */
#define ICPROP_YANG       2   /* "ic_file" and Yang et al. [2000]         */
#define ICPROP_MIE        3   /* "ic_file" and "ic_properties mie"        */
#define ICPROP_FILE       4   /* "ic_file" and "ic_properties <filename>" */
#define ICPROP_EXPLICIT   5   /* "ic_files"                               */


#define PROCESS_NONE      0
#define PROCESS_SUM       1
#define PROCESS_INT       2

#define OUTCAL_ABSOLUTE        0
#define OUTCAL_THERMAL         1
#define OUTCAL_TRANSMITTANCE   2
#define OUTCAL_REFLECTIVITY    3
#define OUTCAL_BRIGHTNESS      4

#define ISCCP_WATER  0
#define ISCCP_ICE    1

#define MAXAWVN 47


/* minimum number of photons for each band*/
#define MIN_MCPHOTONS 100


/* Monte Carlo input STRUCTURE */

typedef struct {
  int     photons;

  int     Nx_sample;
  int     Ny_sample;

  int     Nx_cloud;
  int     Ny_cloud;
  int     Nz_cloud;

  int     passback3D;
  int     ipa;         /* MYSTIC in IPA mode                 */
  int     escape;      /* escape probabilities for radiances */
  float   truncate;    /* truncate phase function            */

  int     bcond;       /* boundary conditions                */

  int     absorption;  /* calculate absorption for each box  */ 

  int     momcld;
  char    **filename;
} mc_inp_struct;


/* Raditive Transfer Equation STRUCTURE */
/* See disort.doc for description of variables */

typedef struct {
  int     deltam;
  int     fisot;
  int     ichap;  
  int     nrefrac;  
  int     nstr;  
  int     nscat;
  int     maxphi;
  int     maxumu;
  int     nphi;
  int     numu;
  int     solver;
  int     nprndis;
  int     reverse;
  int*    prndis;
  int     pol_aziorder;
  int     pol_nstokes;
  char*   pol_quad_type;
  char    pol_stokes[4];
  int     pol_src_code;
  double  pol_max_delta_tau;
  float*  phi;
  float*  umu;
  mc_inp_struct mc;
} rte_struct;


/* Cloud output STRUCTURE */

typedef struct {
  /* quantities as defined by the user */
  float *lwc;        /* altitude profile of the liquid water content */
  float *effr;       /* altitude profile of the effective radius     */

  /* layer quantities */
  float *lwc_layer;  /* altitude profile of the liquid water content */
  float *effr_layer; /* altitude profile of the effective radius     */

  int   alloc;      /* flag, if memory is allocated                 */
} cld_microphys_struct;


/********************************************************/
/* optical properties of aerosol, water, and ice clouds */
/********************************************************/

typedef struct {
  float **dtau;     /* layer optical thickness [wavelength][layer]         */

  float **ff;       /* f  for double HG [wavelength][layer]                */
  float **g1;       /* g1 for double HG [wavelength][layer]                */
  float **g2;       /* g2 for double HG [wavelength][layer]                */

  float **ssa;      /* layer single scattering albedo [wavelength][layer]  */
  float **fdel;     /* delta-m scaling factor                              */

  int   nlam;       /* number of wavelengths                               */
	
  float ***moment;  /* moments of the phase function for each              */
                    /* wavelength and layer                                */

  int   **nmom;     /* number of moments for each wavelength and layer     */

  int alloc;        /* flag, if memory is allocated;                       */
} optprop_struct;


typedef struct {
  /* the following quantities are defined for each wavelength, */
  /* on an equidistant grid of effective radii,                */
  /* r0, r0+dr, r0+2*dr, ..., r0 + (nreff-1)*dr                */

  double **extinc;    /* extinction coefficient           */
  double **albedo;    /* single scattering albedo         */
  float  ***legen;    /* Legendre moments                 */
  int    **nleg;      /* number of Legendre moments       */
  double  *r0;        /* start radius for each wavelength */
  double  *dr;        /* wavelength step                  */
  int     *nreff;     /* number of effective radii        */ 
  
  int   alloc;        /* flag, if memory is allocated     */
} cld_ssprop_struct;


typedef struct {

  float *zd;       /* level altitudes                              */
  int   nlev;      /* number of levels                             */

  cld_microphys_struct microphys;  /* microphysical properties     */
  optprop_struct       optprop;    /* optical properties           */
  cld_ssprop_struct    ssprop;     /* single scattering properties */

  int     newsiz;
} cld_out_struct;



/* Aerosol input STRUCTURE */

typedef struct {
  int     standard;  /* Shettle standard aerosol defined    */
  int     spec;      /* specific aerosol properties defined */

  char **filename;   /* array of filenames */
  int haze;
  int vulcan;
  int seasn;
  float alpha;
  float beta;
  float visibility;
  
  float re;          /* refractive index            */
  float im;          /* (independent of wavelength) */

  float tau550;

  float scale_ssa;
  float scale_tau;
  float set_gg;
  float set_ssa;
  float set_tau;
} aer_inp_struct;

/* Aerosol output STRUCTURE */

typedef struct {
  float   visibility;

  float *zd;       /* level altitudes                  */
  int   nlev;      /* number of levels                 */

  float *zd_alt;       /* level altitudes, starting at 'altitude'   */
  int   nlev_alt;      /* number of levels, starting at 'altitude'  */

  /* optical properties profile */
  optprop_struct optprop;

  float   *aer_pro;

  double   *xsiz;    /* size distribution              */
  double   *ysiz;
  int      nsiz;

  float   *re_t;     /* real and imaginary part of the */
  float   *im_t;     /* refractive index               */
} aer_out_struct;



/* RPV (Rahman, Pinty, Verstraete) structure */

typedef struct {
  float rho0;
  float k;
  float theta;
} rpv_inp_struct;


/* Cox and Munk structure */

typedef struct {
  float u10;
  float pcl;
  float sal;
} cm_inp_struct;


/* Cloud input STRUCTURE */

typedef struct {
  int     layer;

  float   tau;  
  float   gg;  
  float   ssa;  

  float   tau550;  

  float scale_gg;
  float scale_ssa;

  int properties;
  int interpolate;
  int habit;
} cld_inp_struct;



/* RPV (Rahman, Pinty, Verstraete) structure */
/* to store wavelength dependent BRDF        */
typedef struct {
  int   n;

  float *lambda;
  float *rho0;     /* input wavelength grid    */
  float *k;
  float *theta;

  float *rho0_t;   /* internal wavelength grid */
  float *k_t;
  float *theta_t;
} rpv_out_struct;



/* Array of RPV (Rahman, Pinty, Verstraete) structures */
/* for different surfaces, addressed by a label        */
typedef struct {
  int   n;

  char  **label;
  char  **filename;

  rpv_out_struct *rpv;
} surfaces_out_struct;



/* MYSTIC uses profiles in reverted order (the lowest level is 0) */

typedef struct {
  float       *dtair;           /* free'ed */
  float       *dtmol;           /* free'ed */
  float       *dtaer;           /* free'ed */
  float       *dtcld;           /* free'ed */
  float       *dtice;           /* free'ed */
  float       *omaer;           /* free'ed */
  float       *omcld;           /* free'ed */
  float       *omice;           /* free'ed */

  float       *g1aer;
  float       *g2aer;
  float       *ffaer;

  float       *g1cld;
  float       *g2cld;
  float       *ffcld;

  float       *g1ice;
  float       *g2ice;
  float       *ffice;

  float       **momaer;
  int         *nmomaer;

  float       *z;
  float       *temper;

  float       *recld;
  float       *lwccld;
} mc_out_struct;

typedef struct {
  float tautot;
  int lctop;
  int type;
  float ref;
} sssi_out_struct;

/* Wavelength input grid STRUCTURE */

typedef struct {
  float start;
  float end;

  int start_index;
  int end_index;
} wl_inp_struct;

/* Wavelength output grid STRUCTURE */

typedef struct {
  float start;
  float end;

  int   type;

  int   nlambda_h;
  int   nlambda_s;
  int   nlambda_t;

  float *lambda_h;
  float *lambda_s;
  float *lambda_t;

  int   *map_e2h;

  float *fbeam;
  float *filter;

  float *wvnmlo_t;
  float *wvnmhi_t;

  float *wvnmlo_h;
  float *wvnmhi_h;

  int  nlambda_rte_lower;
  int  nlambda_rte_upper;

  float *pfraction;
} wl_out_struct;

/* Atmosphere input data STRUCTURE */

typedef struct {
  char  **filename;    /* Filename of density profile, input using dens_file */
  float *column;       /* Total column of a profile, input using dens_column */
  float *scale_factor; /* Factor to scale profile with, calculated           */
  int   *unit;         /* Unit of column, input using dens_column            */

  float *zout_sur;     /* output altitudes, relative to the surface          */
  int    nzout;        /* number of output altitudes                         */

  float  albedo;

  float  altitude;
  float  altitude_dz;

  float  sza;
  float  phi0;
} atm_inp_struct;

/* Atmosphere output data STRUCTURE */

typedef struct {

  float *press;
  float *refind;
  float *rh;
  float *temper;

  float **dens;          /* density profiles */
  float ***denstab;      /* density profiles as function of altitude and sza */
  float **denstab_amf;   /* density profile as function of altitude and sza, for sdisort */

  float *sza_denstab;    /* Ary for the sza in denstab */
  
  int    denstab_id;      /* Density id for denstab, MOL_* */
  int    nsza_denstab;    /* Number of sza in denstab */

} atm_microphys_struct;


typedef struct {
  
  float *tau_rayleigh;

  double ***tau_rayleigh_t;    /* wavelength-dependent Rayleigh scattering optical depth   */
  double ***tau_molabs_t;      /* wavelength-dependent molecular absorption optical depth  */
  double ***tau_molabs_md_t;   /* wavelength-dependent molecular absorption optical depth  */
			       /* with density-matrix absorption subtracted. For AMF       */
			       /* calculations                                             */
                               /* format: tau_molabs_t[lc][iv][iq]                         */
                               /* ??? this is not consistent with tau_molabs_user_t[][]    */
                               /* ??? and should be changed accordingly                    */

  /* the following are explicitely defined by the user and do not depend on subband iq     */
  double  *tau_molabs_user;    /* monochromatic molecular absorption optical depth         */
  double **tau_molabs_user_t;  /* wavelength-dependent molecular absorption optical depth  */
                               /* format: tau_molabs_user_t[iv][lc]                        */
} atm_optprop_struct;


typedef struct {
  char* pol_scat_files;
  float *cdento;

  float *temper_out;

  atm_microphys_struct microphys;
  atm_optprop_struct   optprop;

  int   molabs;               /* molecular absorption flag         */

  float *zd;
  int   nlev;
  int   nlyr;

  float  altitude;     /* Same as altitude from input structure is set.
			  Otherwise obtained from atmosphere input file */
  float *zout_sur;     /* output altitudes, relative to the surface */
  float *zout_sea;     /* output altitudes, relative to sea level   */
  int    nzout;        /* number of output altitudes                */

  float *zd_common;
  int    nlev_common;
  int    nlyr_common;

  double **wght_t;            /* ck quadrature weight              */
  int    *nq_t;               /* number of ck quadrature weights   */

  float *file_albedo;
  float *file_lambda_albedo;
  float *albedo_t;

  float *file_phi0;
  float *file_sza;
  float *file_lambda_sza;
  float *sza_t;
  float *phi0_t;

  int    file_nlambda_albedo;
  int    nlambda_albedo;

  int    file_nlambda_sza;
  int    nlambda_sza;

  int    nmom;  
} atm_out_struct;

/*******************************************************************/
/* The ck structure holds all the information required for         */
/* a correlated-k approximation.                                   */
/* Components:                                                     */
/*    scheme               CK_KATO, CK_FU                          */
/*    wvlc [1 ... n_wvl]   central wavelength [nm]                 */                   
/*    wvl1 [1 ... n_wvl]   start wavelength [nm] for each band     */    
/*    wvl2 [1 ... n_wvl]   end wavelength [nm] for each band       */    
/*    n_wvl                number of wavelength bands              */
/*    comp [1 ... n_comp][1 ... n_wvl]                             */
/*                         monochromatic cross sections bands      */
/*                         which don't require c-k but only a      */
/*                         monochromatic calculation               */
/*                         (e.g. ozone in the UV/VIS)              */
/*    wght [1 ... n_comp][1 ... n_wvl]                             */
/*                         number of quadrature points             */
/*******************************************************************/

typedef struct {   /* correlated-k structure */
  int scheme;

  double *wvlc;
  double *wvl1;
  double *wvl2;
  int n_wvl;

  double **comp;
  int    **wght;

  void *x_thermal;  /* photon fraction; x_thermal[iv][iq1][iq2][...] */
  void *x_solar;    /* photon fraction; x_solar  [iv][iq1][iq2][...] */

  int n_comp;
} ck_struct;

/* Cross section output data STRUCTURE */

typedef struct {
  char  **filename;  /*  Filename of cross section versus wavelength, input using crs_file  */
  float ***crs;      /* 3D ary, for cross section[temperature (or layer), type, wavelength] */
  float **crs_amf;   /* 2D ary for amf calculations, needed by sdisort */
} crs_out_struct;

typedef struct {
  float** kabs; /* 2D ary, for absorption coefficient[temperature (or layer), wavelength] */
} kabs_out_struct;

typedef struct {
  double **crs;      /* cross section [layer, quadrature point] */
  double *weight;    /* weight [quadrature point]               */
  int ngauss;        /* number of quadrature points             */
  int nlev;
} ck_profile;

typedef struct {
  ck_profile **profile;  /* 2D ary, for ck_profile [type, band] */
} crs_ck_out_struct;


			    
/* INPUT STRUCTURE */

typedef struct {
  int quiet;
  int verbose;

  char         **filename;   /* array of filenames */
  
  float        cloudcover;
  int          wcipa;        /* independent pixel for water clouds */
  int          icipa;        /* independent pixel for ice clouds   */
  int          convolve;
  int          spline;
  int          header;
  int          flexstor;
  int          source;
  double       bandwidth;
  int          rayleigh;
  int          molabs;
  int          absorption;
  int          scattering;
  float        spline_lambda_0;
  float        spline_lambda_1;
  float        spline_lambda_step;
  int          day_of_year;
  int          ozone_spectrum;
  float        no2_column;
  int          no2_spectrum;
  int          bro_spectrum;
  int          oclo_spectrum;
  int          hcho_spectrum;
  float        precip_water;
  float        pressure;
  float        temperature;
  float        *mixing_ratio;
  int          calibration;

  int    brdf;
  float  u10;       /* wind speed for Cox and Munk parameterization */

  rte_struct   rte;
  atm_inp_struct atm;
  aer_inp_struct aer;

  rpv_inp_struct rpv;
  cm_inp_struct cm;

  wl_inp_struct  wl;
  cld_inp_struct wc;
  cld_inp_struct ic;
  int            ck_scheme;
  int            ck_h2ocont;

  int            processing;
} input_struct;


/* OUTPUT STRUCTURE */

/* elements whose memory requirements have been free'ed are listed behind */
/* each declaration. Once they are all done rm this comment.              */

typedef struct {
  float       eccentricity_factor;

  float       ozone_column;
  float       ozone_scale_factor;

  float       bro_column;
  float       bro_scale_factor;

  float       no2_column;
  float       no2_scale_factor;

  float       oclo_column;
  float       oclo_scale_factor;

  float       hcho_column;
  float       hcho_scale_factor;

  float       precip_water;
  float       water_scale_factor;

  int         mc_photons;
  double      **mc_photons_t;   /* photon fraction for each wavelength and quadrature point */
  char        *mc_momcldfilename;

  float       *mixing_ratio;

  float       *dtauc;           /* free'ed */
  float       *dtauc_md;        /* free'ed */
  float       *ssalb;
  float       *utau;
  float       **pmom;

  float       surface_temperature;

  /* flux, actinic flux, and intensities on the output grid */ 
  float       **rfldir;     /* direct flux                    */
  float       **rfldn;      /* diffuse downward flux          */
  float       **flup;       /* diffuse upward flux            */ 

  float       **uavgso;     /* direct actinic flux            */
  float       **uavgdn;	    /* diffuse downward actinic flux  */
  float       **uavgup;	    /* diffuse upward actinic flux    */

  float       **uavg;       /* average intensity              */
  float       ***u0u;       /* azimutally averaged intensites */
  float       ****uu;       /* intensities                    */

  /* MYSTIC 3D fields on the output grid */  
  float       ****rfldir3d; 
  float       ****rfldn3d;
  float       ****flup3d;
  float       ****radiance3d;
  float       ****abs3d;

  /* flux, actinic flux, and intensities on the internal transmittance grid */ 
  float       **rfldir_t;  /* direct flux                    */
  float       **rfldn_t;   /* diffuse downward flux          */
  float       **flup_t;	   /* diffuse upward flux            */
			                                       
  float       **uavgso_t;  /* direct actinic flux            */
  float       **uavgdn_t;  /* diffuse downward actinic flux  */
  float       **uavgup_t;  /* diffuse upward actinic flux    */
			                                       
  float       **uavg_t;	   /* average intensity              */
  float       ***u0u_t;	   /* azimutally averaged intensites */
  float       ****uu_t;	   /* intensities                    */

  /* MYSTIC 3D fields on the internal transmittance grid */  
  float       ****rfldir3d_t;
  float       ****rfldn3d_t;
  float       ****flup3d_t;
  float       ****radiance3d_t;
  float       ****abs3d_t;


  /* PolRadtran flux and intensities on the output grid */ 
  float       ***down_flux;     /* free'ed */
  float       ***up_flux;       /* free'ed */
  float       *****down_rad;    /* free'ed */
  float       *****up_rad;      /* free'ed */

  /* PolRadtran flux and intensities on the internal transmittance grid */ 
  float       ***down_flux_t;   /* free'ed */
  float       ***up_flux_t;     /* free'ed */
  float       *****down_rad_t;  /* free'ed */
  float       *****up_rad_t;    /* free'ed */


  float       *mu_values;       /* free'ed */
  float       *sza_h;           /* free'ed */

  atm_out_struct      atm;
  aer_out_struct      aer;
  crs_out_struct      crs;
  rpv_out_struct      rpv;
  surfaces_out_struct surf;
  kabs_out_struct     kabs;
  crs_ck_out_struct   crs_ck;

  cld_out_struct    wc;
  cld_out_struct    *wcipa;
  cld_out_struct    ic;
  cld_out_struct    *icipa;

  mc_out_struct     mc;
  sssi_out_struct   sssi;

  float             *ipaweight;
  int               nipa;

  int               print_phi;

  wl_out_struct     wl;
  ck_struct         ck;
} output_struct;

int uvspec(input_struct input, output_struct *output);
int uvspec_check(input_struct input);


/* include all uvspec header files */
#include "albedo.h"
#include "aerosol.h"
#include "ancillary.h"
#include "atmosphere.h"
#include "ck.h"
#include "cloud.h"
#include "extraterrestrial.h"
#include "ipa.h"
#include "molecular.h"
#include "mystic.h"
#include "redistribute.h"
#include "solve_rte.h"
#include "sza.h"
#include "yang56.h"
#endif
