/*--------------------------------------------------------------------
 * $Id: uvspec_lex.l,v 1.95 2003/11/27 17:48:13 pa2h Exp $
 * 
 * This file is part of libRadtran.
 * Copyright (c) 1997-2001 by Arve Kylling and Bernhard Mayer.
 * 
 * ######### Contact info: http://www.libradtran.org #########
 *
 * This program is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License   
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.        
 * 
 * This program is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of  
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the   
 * GNU General Public License for more details.                    
 * 
 * You should have received a copy of the GNU General Public License          
 * along with this program; if not, write to the Free Software                
 * Foundation, Inc., 59 Temple Place - Suite 330, 
 * Boston, MA 02111-1307, USA.
 *--------------------------------------------------------------------*/

/* the "incl" state is used for picking up the name
 * of an include file
 */
%x include

%{

#include <getopt.h>
#include <math.h>
#include <stdlib.h>
#include <string.h>
#include "uvspec.h"
#include <time.h>
#include "../src_c/ascii.h"
#include "../src_c/r250.h"

static float  yytext2float(char* yytext, int which_token, int max_token);
static float* yytext2floats(char* yytext, int nary);
static double yytext2double(char* yytext, int which_token, int max_token);
static int    yytext2int(char* yytext, int which_token, int max_token);
static int*   yytext2integers(char* yytext, int nary);
static char*  yytext2string(char* yytext, int which_token, int max_token);
static int    yytext2ntokens(char* yytext);
static int    yytextCstring(char *dest, char* yytext, int which_token, int max_token);
static void free_output(input_struct input, output_struct *output);
static int print_output(input_struct input, output_struct *output);


char* s;
int ntokens;
int mol_id;
float tmpfloat;
int   tmpint;
float* tmp;
int*   tmpi;
int ierror=0;
int start_condition = INITIAL;
int line_number;
input_struct Input;
output_struct Output;

#define MAX_INCLUDE_DEPTH 10
YY_BUFFER_STATE include_stack[MAX_INCLUDE_DEPTH];
int include_stack_ptr = 0;

%}

%option noyywrap

DIGIT    [0-9]
SPACE    [ \t]
CCHAR    [#]
COMMENT  ({CCHAR}.*)
INTEGER  {DIGIT}+
SPACE_INTEGER {SPACE}+{INTEGER}
INTEGERS {SPACE_INTEGER}+ 
EXPON   ([DdEe][+-]?{DIGIT}+)
FLOAT    ((({DIGIT}+"."{DIGIT}*)|({DIGIT}+"."{DIGIT}*{EXPON}))|({INTEGER}))
SPACE_FLOAT {SPACE}+{FLOAT}
FLOATS   {SPACE_FLOAT}+ 
SIGN     [-+]
SIGNED_FLOAT {SIGN}?{FLOAT}
SPACE_SIGNED_FLOAT {SPACE}+{SIGNED_FLOAT}
SIGNED_FLOATS {SPACE_SIGNED_FLOAT}+
SIGNED_INTEGER {SIGN}?{INTEGER}
WORD     [_\.\/A-Za-z][_\.\/A-Za-z0-9\-]*
SPACE_WORD {SPACE}+{WORD}

%%

  /* These are the allowed input lines in the uvspec input file 
     The input line specifications must come before the eating of
     blanks, otherwise things get messed up 
     Obviously this whole lot could have been handled more properly
     using bison etc., but then rather few would know what was going 
     on??? 
   */

aerosol_angstrom{SIGNED_FLOATS}     {s = (char *) calloc (strlen(yytext)+1,sizeof(char));
				     strcpy (s, yytext);
				     Input.aer.alpha    = yytext2float(s,2,3); 
                                     Input.aer.beta     = yytext2float(yytext,3,3);
				     free(s);
                                     Input.aer.spec++;}
aerosol_default                     {Input.aer.haze = 1;
                                     Input.aer.vulcan = 1; 
				     Input.aer.seasn = 1;
                                     Input.aer.visibility = 50.;}
aerosol_files{SPACE}+{WORD}         {yytextCstring (Input.aer.filename[FN_AER_FILES],
						    yytext, 2, 2);
                                     Input.aer.spec++;}
aerosol_gg_file{SPACE}+{WORD}       {yytextCstring (Input.aer.filename[FN_AER_GG],
						    yytext, 2, 2);
                                     Input.aer.spec++;}
aerosol_haze{SPACE}+{INTEGER}       {Input.aer.haze     = yytext2int(yytext,2,2); }
aerosol_moments_file{SPACE}+{WORD}  {yytextCstring (Input.aer.filename[FN_AER_MOM],
						    yytext, 2, 2);
                                     Input.aer.spec++;}
aerosol_refrac_file{SPACE}+{WORD}   {yytextCstring (Input.aer.filename[FN_AER_REF],
						    yytext, 2, 2);
                                     Input.aer.spec++;}
aerosol_refrac_index{SPACE_FLOAT}{SPACE_FLOAT} {s = (char *) calloc (strlen(yytext)+1,sizeof(char));
                                     strcpy (s, yytext);
                                     tmp   = yytext2floats(yytext, 2); 
                                     Input.aer.re = tmp[0];
				     Input.aer.im = tmp[1];
				     free(tmp); free(s); 
                                     Input.aer.spec++;}
aerosol_scale_ssa{SPACE}+{FLOAT}    {Input.aer.scale_ssa= yytext2float(yytext,2,2); 
                                     Input.aer.spec++;}
aerosol_scale_tau{SPACE}+{FLOAT}    {Input.aer.scale_tau= yytext2float(yytext,2,2); 
                                     Input.aer.spec++;}
aerosol_set_gg{SPACE}+{SIGNED_FLOAT} {Input.aer.set_gg   = yytext2float(yytext,2,2); 
                                     Input.aer.spec++;}
aerosol_set_ssa{SPACE}+{FLOAT}      {Input.aer.set_ssa  = yytext2float(yytext,2,2); 
                                     Input.aer.spec++;}
aerosol_set_tau{SPACE}+{FLOAT}      {Input.aer.set_tau  = yytext2float(yytext,2,2); 
                                     Input.aer.spec++;}
aerosol_set_tau550{SPACE}+{FLOAT}   {Input.aer.tau550 = yytext2float(yytext,2,2); }
aerosol_season{SPACE}+{INTEGER}     {Input.aer.seasn    = yytext2int(yytext,2,2); 
                                     Input.aer.spec++;}
aerosol_sizedist_file{SPACE}+{WORD} {yytextCstring (Input.aer.filename[FN_AER_SIZ], 
						    yytext, 2, 2);
                                     Input.aer.spec++;}
aerosol_ssa_file{SPACE}+{WORD}      {yytextCstring (Input.aer.filename[FN_AER_SSA], 
						    yytext, 2, 2);
                                     Input.aer.spec++;}
aerosol_tau_file{SPACE}+{WORD}      {yytextCstring (Input.aer.filename[FN_AER_TAU],
						    yytext, 2, 2);
                                     Input.aer.spec++;}
aerosol_visibility{SPACE}+{FLOAT}   {Input.aer.visibility    = yytext2float(yytext,2,2); }
aerosol_vulcan{SPACE}+{INTEGER}     {Input.aer.vulcan   = yytext2int(yytext,2,2); }
albedo{SPACE}+{FLOAT}               {Input.atm.albedo   = yytext2float(yytext,2,2); }
albedo_file{SPACE}+{WORD}           {yytextCstring (Input.filename[FN_ALBEDO],
						    yytext, 2, 2);
                                     Input.atm.albedo   = -1.;}
altitude{SPACE}+{SIGNED_FLOAT}{SPACE_SIGNED_FLOAT}* {
                                     s = (char *) calloc (strlen(yytext)+1,sizeof(char));
				     strcpy (s, yytext);
                                     ntokens = yytext2ntokens(s);
                                     free(s);            
				     s =  yytext2string(yytext,2,ntokens+1);
                                     Input.atm.altitude = yytext2float(yytext,2,ntokens+1); 
                                     if ( ntokens == 2 ) {
                                        Input.atm.altitude_dz = yytext2float(yytext,3,ntokens+1); 
				     }
                                     }
angstrom{SIGNED_FLOATS}             {s = (char *) calloc (strlen(yytext)+1,sizeof(char));
				     strcpy (s, yytext);
				     Input.aer.alpha    = yytext2float(s,2,3); 
                                     Input.aer.beta     = yytext2float(yytext,3,3);
				     free(s);
                                     Input.aer.spec++;}
atmosphere_file{SPACE}+{WORD}       {yytextCstring(Input.filename[FN_ATMOSPHERE],yytext,2,2); }
brightness                          {Input.calibration = OUTCAL_BRIGHTNESS;}
ch4_mixing_ratio{SPACE}+{FLOAT}     {Input.mixing_ratio[MX_CH4]   = yytext2float(yytext,2,2); }
co2_mixing_ratio{SPACE}+{FLOAT}     {Input.mixing_ratio[MX_CO2]   = yytext2float(yytext,2,2); }
correlated_k{SPACE}+{WORD}          {s = yytext2string(yytext,2,2);
				     if (strlen(s)==8&&strncasecmp("kato2.96",s,8)==0)    {Input.ck_scheme = CK_KATO2_96;}
				     else if (strlen(s)==5&&strncasecmp("kato2",s,5)==0) {Input.ck_scheme = CK_KATO2;}
				     else if (strlen(s)==4&&strncasecmp("kato",s,4)==0) {Input.ck_scheme = CK_KATO;}
				     else if (strncasecmp("fu",s,2)==0) {Input.ck_scheme = CK_FU;}
				     else if (strncasecmp("avhrr_kratz",s,2)==0) {Input.ck_scheme = CK_AVHRR_KRATZ;}
				     else if (strncasecmp("sbdart",s,2)==0) {Input.ck_scheme = CK_SBDART;}
				     else {Input.ck_scheme = CK_GENERIC; 
				           strcpy (Input.filename[FN_CK_GENERIC], s);}
				     free(s);}
cox_and_munk_u10{SPACE}+{FLOAT}     {Input.cm.u10 = yytext2float(yytext,2,2); }
cox_and_munk_pcl{SPACE}+{FLOAT}     {Input.cm.pcl = yytext2float(yytext,2,2); }
cox_and_munk_sal{SPACE}+{FLOAT}     {Input.cm.sal = yytext2float(yytext,2,2); }
crs_file{SPACE}+{WORD}{SPACE}+{WORD} {
                                     s = yytext2string(yytext,2,4);
				     if (strncmp("O3",s,2)==0)       {yytextCstring(Output.crs.filename[MOL_O3],yytext,3,4);}
				     else if (strncmp("O2",s,2)==0)  {yytextCstring(Output.crs.filename[MOL_O2],yytext,3,4);}
				     else if (strncmp("H2O",s,3)==0) {yytextCstring(Output.crs.filename[MOL_H2O],yytext,3,4);}
				     else if (strncmp("CO2",s,3)==0) {yytextCstring(Output.crs.filename[MOL_CO2],yytext,3,4);}
				     else if (strncmp("NO2",s,3)==0) {yytextCstring(Output.crs.filename[MOL_NO2],yytext,3,4);}
				     else if (strncmp("BRO",s,3)==0) {yytextCstring(Output.crs.filename[MOL_BRO],yytext,3,4);}
				     else if (strncmp("OCLO",s,4)==0) {yytextCstring(Output.crs.filename[MOL_OCLO],yytext,3,4);}
				     else if (strncmp("HCHO",s,4)==0) {yytextCstring(Output.crs.filename[MOL_HCHO],yytext,3,4);}
                                     free(s);
                                     }
data_files_path{SPACE}+{WORD}       {yytextCstring (Input.filename[FN_PATH],
						    yytext, 2, 2); 
                                     /* append slash */
                                     if (Input.filename[FN_PATH][strlen(Input.filename[FN_PATH])-1]!= '/')
				       strcat (Input.filename[FN_PATH], "/");}
day_of_year{SPACE}+{INTEGER}        {Input.day_of_year  = yytext2int(yytext,2,2); }
deltam{SPACE}+{WORD}                {s =  yytext2string(yytext,2,2);
				     if (strncmp("on",s,2)==0) {Input.rte.deltam = 1;}
				     else if (strncmp(s,"off",3)==0) {Input.rte.deltam = 0;}
				     free(s);}
dens_column{SPACE}+{WORD}{SPACE}+{FLOAT}{SPACE_WORD}? {s = (char *) calloc (strlen(yytext)+1,sizeof(char));
				     strcpy (s, yytext);
                                     ntokens = yytext2ntokens(s);
                                     free(s);            
				     s =  yytext2string(yytext,2,ntokens+1);
				     if (strncmp("O3",s,2)==0)       {Input.atm.column[MOL_O3]=yytext2float(yytext,3,ntokens+1); mol_id = MOL_O3; }
                                     else if (strncmp("O2",s,2)==0)  {Input.atm.column[MOL_O2]=yytext2float(yytext,3,ntokens+1); mol_id = MOL_O2; }
                                     else if (strncmp("H2O",s,3)==0) {Input.atm.column[MOL_H2O]=yytext2float(yytext,3,ntokens+1); mol_id = MOL_H2O; }
                                     else if (strncmp("CO2",s,3)==0) {Input.atm.column[MOL_CO2]=yytext2float(yytext,3,ntokens+1); mol_id = MOL_CO2; }
                                     else if (strncmp("NO2",s,3)==0) {Input.atm.column[MOL_NO2]=yytext2float(yytext,3,ntokens+1); mol_id = MOL_NO2; }
                                     else if (strncmp("BRO",s,3)==0) {Input.atm.column[MOL_BRO]=yytext2float(yytext,3,ntokens+1); mol_id = MOL_BRO; }
                                     else if (strncmp("OCLO",s,4)==0) {Input.atm.column[MOL_OCLO]=yytext2float(yytext,3,ntokens+1); mol_id = MOL_OCLO; }
                                     else if (strncmp("HCHO",s,4)==0) {Input.atm.column[MOL_HCHO]=yytext2float(yytext,3,ntokens+1); mol_id = MOL_HCHO; }
                                     free(s);            

				     if ( ntokens == 3 ) {
   				        s =  yytext2string(yytext,4,ntokens+1);
				        if (strncmp("DU",s,2)==0)        {Input.atm.unit[mol_id]= DU; }
				        else if (strncmp("CM_2",s,4)==0) {Input.atm.unit[mol_id]= CM_2; }
                                        free(s);            
                                     }
                                     }
dens_file{SPACE}+{WORD}{SPACE}+{WORD} {
                                     s = yytext2string(yytext,2,4);
				     if (strncmp("O3",s,2)==0)       {yytextCstring(Input.atm.filename[MOL_O3],yytext,3,4);}
				     else if (strncmp("O2",s,2)==0)  {yytextCstring(Input.atm.filename[MOL_O2],yytext,3,4);}
				     else if (strncmp("H2O",s,3)==0) {yytextCstring(Input.atm.filename[MOL_H2O],yytext,3,4);}
				     else if (strncmp("CO2",s,3)==0) {yytextCstring(Input.atm.filename[MOL_CO2],yytext,3,4);}
				     else if (strncmp("NO2",s,3)==0) {yytextCstring(Input.atm.filename[MOL_NO2],yytext,3,4);}
				     else if (strncmp("BRO",s,3)==0) {yytextCstring(Input.atm.filename[MOL_BRO],yytext,3,4);}
				     else if (strncmp("OCLO",s,4)==0) {yytextCstring(Input.atm.filename[MOL_OCLO],yytext,3,4);}
				     else if (strncmp("HCHO",s,4)==0) {yytextCstring(Input.atm.filename[MOL_HCHO],yytext,3,4);}
                                     free(s);
                                     }
f11_mixing_ratio{SPACE}+{FLOAT}     {Input.mixing_ratio[MX_F11]   = yytext2float(yytext,2,2); }
f12_mixing_ratio{SPACE}+{FLOAT}     {Input.mixing_ratio[MX_F12]   = yytext2float(yytext,2,2); }
f22_mixing_ratio{SPACE}+{FLOAT}     {Input.mixing_ratio[MX_F22]   = yytext2float(yytext,2,2); }
filter_function_file{SPACE}+{WORD}  {yytextCstring(Input.filename[FN_FILTERFUNCTION],yytext,2,2); }
fisot                               {Input.rte.fisot = 1; }
flexstor                            {Input.flexstor  = 1; }
h2o_continuum{SPACE}+{WORD}         {s = yytext2string(yytext,2,2);
                                     if (strncasecmp("on",s,2)==0) 
				       Input.ck_h2ocont = 1;
				     else {if (strncasecmp(s,"off",3)==0) 
				       Input.ck_h2ocont = 0;}
				     free(s);}
h2o_mixing_ratio{SPACE}+{FLOAT}     {Input.mixing_ratio[MX_H2O]   = yytext2float(yytext,2,2); }
h2o_precip{SPACE}+{FLOAT}           {Input.precip_water   = yytext2float(yytext,2,2); }
header                              {Input.header    = 1; }
ichap{SPACE}+{INTEGER}              {Input.rte.ichap = yytext2int(yytext,2,2);}
ic_file{SPACE}+{WORD}               {yytextCstring (Input.filename[FN_ICLOUD],
						    yytext, 2, 2);}
ic_files{SPACE}+{WORD}              {yytextCstring (Input.filename[FN_ICLOUD_FILES], 
						    yytext, 2, 2);
                                     Input.ic.properties = ICPROP_EXPLICIT;}
ic_habit{SPACE}+{WORD}              {Input.ic.habit = -999;
                                     s =  yytext2string(yytext,2,2);       /*agf*/
                                     if (strncasecmp("solid-column",s,12)==0)
				       Input.ic.habit = IC_HABIT_SOLIDCOLUMN;
				     else {
				       if (strncasecmp("hollow-column",s,13)==0)
					 Input.ic.habit = IC_HABIT_HOLLOWCOLUMN;
				       else{ 
				         if (strncasecmp("rough-aggregate",s,15)==0)
					   Input.ic.habit = IC_HABIT_AGGREGATE;
					 else{
					   if (strncasecmp("rosette-4",s,9)==0)
					     Input.ic.habit = IC_HABIT_ROSETTE4;
					   else{
					     if (strncasecmp("rosette-6",s,9)==0)
					       Input.ic.habit = IC_HABIT_ROSETTE6;
					     else{
					       if (strncasecmp("plate",s,5)==0)
					         Input.ic.habit = IC_HABIT_PLATE;
					     }
					   }
					 }
				       }
				     }
				    }
ic_ipa_files{SPACE}+{WORD}          {yytextCstring (Input.filename[FN_ICLOUD_IPA],
						    yytext, 2, 2);
                                     Input.icipa=1;
                                     Input.ic.properties = ICPROP_FU;}
ic_layer                            {Input.ic.layer=1;}
ic_properties{SPACE}+{WORD}         {s =  yytext2string(yytext,2,2);
                                     if (strncmp("fu",s,2)==0)
				       Input.ic.properties = ICPROP_FU;
				     else {
				       if (strncmp("mie",s,3)==0)
					 Input.ic.properties = ICPROP_MIE;
				       else {
				         if (strncmp("yang",s,4)==0)
					   Input.ic.properties = ICPROP_YANG;
					 else  {
					   Input.ic.properties = ICPROP_FILE;
					   strcpy (Input.filename[FN_ICLOUD_PROP], s);
                                         }
				       }
				     }
                                    }
ic_properties_interpolate           {Input.ic.interpolate=1; }
ic_scale_gg{SPACE}+{FLOAT}          {Input.ic.scale_gg   = yytext2float(yytext,2,2); }
ic_scale_ssa{SPACE}+{FLOAT}         {Input.ic.scale_ssa  = yytext2float(yytext,2,2); }
ic_set_gg{SPACE}+{FLOAT}            {Input.ic.gg     = yytext2float(yytext,2,2); }
ic_set_ssa{SPACE}+{FLOAT}           {Input.ic.ssa    = yytext2float(yytext,2,2); }
ic_set_tau{SPACE}+{FLOAT}           {Input.ic.tau    = yytext2float(yytext,2,2); }
ic_set_tau550{SPACE}+{FLOAT}        {Input.ic.tau550 = yytext2float(yytext,2,2); }
include                             BEGIN(include);
mc_absorption                       {Input.rte.mc.absorption = MCABS_ABSORPTION; }
mc_actinic                          {Input.rte.mc.absorption = MCABS_ACTINIC; }
mc_albedo_file{SPACE}+{WORD}        {s = yytext2string(yytext,2,2);
				     strcpy (Input.rte.mc.filename[FN_MC_ALBEDO], s);}
mc_basename{SPACE}+{WORD}           {yytextCstring (Input.rte.mc.filename[FN_MC_BASENAME], 
                                                    yytext, 2, 2);}
mc_bcond{SPACE}+{WORD}              {s =  yytext2string(yytext,2,2);
				     if (strncmp("periodic",s,8)==0)
				       Input.rte.mc.bcond = MCBCOND_PERIODIC;
				     else {
				       if (strncmp("mirror",s,6)==0)
				     	 Input.rte.mc.bcond = MCBCOND_MIRROR;
				     }}
mc_cloud_grid{SPACE_INTEGER}{SPACE_INTEGER}{SPACE_INTEGER} {s = (char *) calloc (strlen(yytext)+1,sizeof(char));
                                     strcpy (s, yytext);
                                     tmpi   = yytext2integers(yytext, 3); 
                                     Input.rte.mc.Nx_cloud = tmpi[0];
				     Input.rte.mc.Ny_cloud = tmpi[1];
				     Input.rte.mc.Nz_cloud = tmpi[2];
				     free(tmpi); free(s); }
mc_elevation_file{SPACE}+{WORD}     {s = yytext2string(yytext,2,2);
				     strcpy (Input.rte.mc.filename[FN_MC_ELEVATION], s);}
mc_emission                         {Input.rte.mc.absorption = MCABS_EMISSION; }
mc_escape                           {Input.rte.mc.escape = 1; }
mc_heating                          {Input.rte.mc.absorption = MCABS_HEATING; }
mc_ipa                              {Input.rte.mc.ipa = 1; }
mc_momcld_file{SPACE}+{WORD}        {s =  yytext2string(yytext,2,2);
				     if (strncmp("water",s,5)==0)
				       Input.rte.mc.momcld = MOMCLD_WATER;
				     else {
				       if (strncmp("ice",s,3)==0)
				     	 Input.rte.mc.momcld = MOMCLD_ICE;
				       else {
				     	 Input.rte.mc.momcld = MOMCLD_FILE;
				     	 strcpy (Input.rte.mc.filename[FN_MC_MOMCLD], s);
				       }
				     }}
mc_pcld_file{SPACE}+{WORD}          {s = yytext2string(yytext,2,2);
				     strcpy (Input.rte.mc.filename[FN_MC_PCLD], s);
				     }
mc_photons{SPACE}+{INTEGER}         {Input.rte.mc.photons  = yytext2int(yytext,2,2); }
mc_photons_file{SPACE}+{WORD}       {yytextCstring(Input.rte.mc.filename[FN_MC_PHOTONS],yytext,2,2); }
mc_rpv_file{SPACE}+{WORD}           {s = yytext2string(yytext,2,2);
				     strcpy (Input.rte.mc.filename[FN_MC_RPV], s);}
mc_rpv_type{SPACE}+{WORD}           {s = yytext2string(yytext,2,2);
				     strcpy (Input.rte.mc.filename[FN_MC_RPVTYPE], s);}
mc_sample_grid{SPACE_INTEGER}{SPACE_INTEGER} {s = (char *) calloc (strlen(yytext)+1,sizeof(char));
                                     strcpy (s, yytext);
                                     tmpi   = yytext2integers(yytext, 2); 
                                     Input.rte.mc.Nx_sample = tmpi[0];
				     Input.rte.mc.Ny_sample = tmpi[1];
				     free(tmpi); free(s); }
mc_truncate                         {Input.rte.mc.truncate=0.99;}
mc_truncate{SPACE}+{SIGNED_FLOAT}   {Input.rte.mc.truncate = yytext2float(yytext,2,2); }
mc_wcloud_file{SPACE}+{WORD}        {s = yytext2string(yytext,2,2);
				     strcpy (Input.rte.mc.filename[FN_MC_WCLOUD], s);}
molecular_tau_file{SPACE}+{WORD}    {yytextCstring(Input.filename[FN_MOLABS],yytext,2,2);}
n2o_mixing_ratio{SPACE}+{FLOAT}     {Input.mixing_ratio[MX_N2O]   = yytext2float(yytext,2,2); }
no_absorption                       {Input.absorption=0; Input.molabs=MOLABS_NONE;}
no_molecular_absorption             {Input.molabs=MOLABS_NONE;}
no_rayleigh                         {Input.rayleigh=RAYLEIGH_NONE;}
no_scattering                       {Input.scattering=0;Input.rayleigh=RAYLEIGH_NONE;}
no2_column_du{SPACE}+{SIGNED_FLOAT} {Input.atm.column[MOL_NO2] = yytext2float(yytext,2,2); 
                                     Input.atm.unit[MOL_NO2] = DU;}
no2_column_moleccm-2{SPACE}+{SIGNED_FLOAT} {Input.atm.column[MOL_NO2] = yytext2float(yytext,2,2);
                                     Input.atm.unit[MOL_NO2] = CM_2;}
nrefrac{SPACE}+{INTEGER}            {Input.rte.nrefrac  = yytext2int(yytext,2,2);}
nstr{SPACE}+{INTEGER}               {Input.rte.nstr  = yytext2int(yytext,2,2);}
nscat{SPACE}+{INTEGER}              {Input.rte.nscat  = yytext2int(yytext,2,2); }
o3_crs{SPACE}+{WORD}                {s =  yytext2string(yytext,2,2);
				     if (strncmp("Bass_and_Paur",s,13)==0) {Input.ozone_spectrum = O3_BASSPAUR;}
				     else if (strncmp(s,"Molina",6)==0)    {Input.ozone_spectrum = O3_MOLINA;}
				     else if (strncmp(s,"Daumont",7)==0)   {Input.ozone_spectrum = O3_DAUMONT;}
				     else {fprintf(stderr,"Unknown 'o3_crs %s' on line %d\n", s, line_number+1);
				     ierror++;}
				     free(s);}
ozone_column{SPACE}+{SIGNED_FLOAT}  {Input.atm.column[MOL_O3] = yytext2float(yytext,2,2); 
                                     Input.atm.unit[MOL_O3] = DU;}
output{SPACE}+{WORD}                {s =  yytext2string(yytext,2,2);
				     if (strncmp("integrate",s,9)==0) {Input.processing = PROCESS_INT;}
				     else if (strncmp("sum",s,3)==0)  {Input.processing = PROCESS_SUM;}
                                     else {fprintf(stderr,"Unknown 'processing %s' on line %d\n", s, line_number+1);
				     ierror++;}
				     free(s);}
quiet                               {Input.quiet = 1;}
phi0{SPACE}+{SIGNED_FLOAT}          {Input.atm.phi0     = yytext2float(yytext,2,2); }
polradtran_aziorder{SPACE}+{INTEGER} {Input.rte.pol_aziorder  = yytext2int(yytext,2,2); }
polradtran_max_delta_tau{SPACE}+{SIGNED_FLOAT} {Input.rte.pol_max_delta_tau = (double) yytext2float(yytext,2,2); }
polradtran_nstokes{SPACE}+{INTEGER} {Input.rte.pol_nstokes  = yytext2int(yytext,2,2); }
polradtran_quad_type{SPACE}+{WORD}  {Input.rte.pol_quad_type
				       = yytext2string (yytext, 2, 2);}
polradtran_src_code{SPACE}+{INTEGER} {Input.rte.pol_src_code = yytext2int(yytext,2,2); }
pressure{SPACE}+{SIGNED_FLOAT}      {Input.pressure = yytext2float(yytext,2,2); }
prndis{INTEGERS}                    {s = (char *) calloc (strlen(yytext)+1,sizeof(char));
				     strcpy (s, yytext);
				     Input.rte.nprndis  = yytext2ntokens(s); 
				     Input.rte.prndis   = 
				       yytext2integers(yytext, Input.rte.nprndis); 
				     free(s); }
rayleigh_tau_file{SPACE}+{WORD}     {yytextCstring(Input.filename[FN_RAYLEIGH],yytext,2,2);
                                     Input.rayleigh = RAYLEIGH_FILE;}
reflectivity                        {Input.calibration = OUTCAL_REFLECTIVITY;}
refractive_index_file{SPACE}+{WORD} {yytextCstring (Input.filename[FN_REFIND],
						    yytext, 2, 2);}
reverse                             {Input.rte.reverse    = 1; }
rh_file{SPACE}+{WORD}               {yytextCstring(Input.filename[FN_RH],yytext,2,2);}
rpv_file{SPACE}+{WORD}              {yytextCstring(Input.filename[FN_RPV],yytext,2,2);}
rpv_k{SPACE}+{SIGNED_FLOAT}         {Input.rpv.k     = yytext2float(yytext,2,2); }
rpv_rho0{SPACE}+{SIGNED_FLOAT}      {Input.rpv.rho0  = yytext2float(yytext,2,2); }
rpv_theta{SPACE}+{SIGNED_FLOAT}     {Input.rpv.theta = yytext2float(yytext,2,2); }
rte_solver{SPACE}+{WORD}            {s =  yytext2string(yytext,2,2);
				     if (strncmp("sdisort",s,7)==0) {Input.rte.solver = SDISORT;}
				     else if (strncmp(s,"spsdisort",9)==0) {Input.rte.solver = SPSDISORT;}
				     else if (strncmp(s,"qdisort",7)==0) {Input.rte.solver = QDISORT;}
				     else if (strncmp(s,"disort2",7)==0) {Input.rte.solver = DISORT2;}
				     else if (strncmp(s,"disort",6)==0) {Input.rte.solver = DISORT;}
				     else if (strncmp(s,"sos",3)==0) {Input.rte.solver = SUCORD;}
				     else if (strncmp(s,"polradtran",6)==0) {Input.rte.solver = POLRADTRAN;}
				     else if (strncmp(s,"twostr",6)==0) {Input.rte.solver = TWOSTR;}
				     else if (strncmp(s,"montecarlo",10)==0) {Input.rte.solver = MONTECARLO;}
				     else if (strncmp(s,"tzs",3)==0) {Input.rte.solver = TZS;}
				     else if (strncmp(s,"sssi",4)==0) {Input.rte.solver = SSSI;}
				     else if (strncmp(s,"sss",3)==0) {Input.rte.solver = SSS;}
				     else {fprintf(stderr,"Unknown 'rte_solver %s' on line %d\n", s, line_number+1);
				     ierror++;}
				     free(s);}
slit_function_file{SPACE}+{WORD}    {yytextCstring(Input.filename[FN_SLITFUNCTION],yytext,2,2);
                                     Input.convolve = 1; }
solar_file{SPACE}+{WORD}            {yytextCstring(Input.filename[FN_EXTRATERRESTRIAL],yytext,2,2); }
source{SPACE}+{WORD}                {s =  yytext2string(yytext,2,2);
                                     Input.source=SRC_NONE; 
                                      if (strcasecmp("solar",s)==0) 
					Input.source = SRC_SOLAR;
				      else if (strcasecmp(s,"thermal")==0) {
					Input.source = SRC_THERMAL;
                                        Input.calibration = OUTCAL_THERMAL;
                                      }}
spline{SIGNED_FLOATS}               {s = (char *) calloc (strlen(yytext)+1,sizeof(char));
				     strcpy (s, yytext);
				     Input.spline_lambda_0    = yytext2float(s,2,4); 
				     strcpy (s, yytext);
				     Input.spline_lambda_1    = yytext2float(s,3,4); 
                                     Input.spline_lambda_step = yytext2float(yytext,4,4);
                                     Input.spline = 1;
				     free(s);}
spline_file{SPACE}+{WORD}           {yytextCstring(Input.filename[FN_SPLINE],yytext,2,2);
                                     Input.spline = 1; }
surface_temperature{SPACE}+{SIGNED_FLOAT}  {Input.temperature = yytext2float(yytext,2,2); }
sza{SPACE}+{SIGNED_FLOAT}           {Input.atm.sza      = yytext2float(yytext,2,2); }
sza_file{SPACE}+{WORD}              {yytextCstring(Input.filename[FN_SZA],yytext,2,2);
                                     Input.atm.sza      = -1000.0;}
thermal_bandwidth{SPACE}+{FLOAT}    {Input.bandwidth = yytext2double(yytext,2,2); }
thermal_bands_file{SPACE}+{WORD}    {yytextCstring(Input.filename[FN_WLBANDS],yytext,2,2); }
transmittance                       {Input.calibration = OUTCAL_TRANSMITTANCE;}
transmittance_wl_file{SPACE}+{WORD} {yytextCstring(Input.filename[FN_WLTRANS],yytext,2,2); }
umu{SIGNED_FLOATS}                  {s = (char *) calloc (strlen(yytext)+1,sizeof(char));
				     strcpy (s, yytext);
				     Input.rte.numu     = yytext2ntokens(s); 
				     Input.rte.maxumu   = Input.rte.numu;
				     Input.rte.umu      = 
				       yytext2floats(yytext, Input.rte.numu); 
				     free(s); }
phi{SIGNED_FLOATS}                  {s = (char *) calloc (strlen(yytext)+1,sizeof(char));
				     strcpy (s, yytext);
				     Input.rte.nphi   = yytext2ntokens(s); 
				     Input.rte.maxphi = Input.rte.nphi;
				     Input.rte.phi    = 
				       yytext2floats(yytext, Input.rte.nphi); 
				     free(s); }
verbose                             {Input.verbose = 1;}
wc_cloudcover{SPACE}+{FLOAT}        {Input.cloudcover= yytext2float(yytext,2,2); 
                                     Input.wcipa=1;}
wc_file{SPACE}+{WORD}               {yytextCstring(Input.filename[FN_WCLOUD],yytext,2,2); }
wc_files{SPACE}+{WORD}              {yytextCstring (Input.filename[FN_WCLOUD_FILES],
						    yytext, 2, 2);
                                     Input.wc.properties = WCPROP_EXPLICIT;}
wc_ipa{SPACE_WORD}                  {Input.wcipa=1;
                                     yytextCstring(Input.filename[FN_WCLOUD_IPA_3D],yytext,2,2);
                                    }
wc_ipa_files{SPACE}+{WORD}          {yytextCstring (Input.filename[FN_WCLOUD_IPA],
						    yytext, 2, 2);
                                     Input.wcipa=1;
                                     Input.wc.properties = WCPROP_HU;}
wc_layer                            {Input.wc.layer=1;}
wc_properties{SPACE}+{WORD}         {s =  yytext2string(yytext,2,2);
                                     if (strncmp("hu",s,2)==0)
				       Input.wc.properties = WCPROP_HU;
				     else {
				       if (strncmp("mie",s,3)==0)
					 Input.wc.properties = WCPROP_MIE;
				       else {
					 Input.wc.properties = WCPROP_FILE;
					 strcpy (Input.filename[FN_WCLOUD_PROP], s);
				       }
				     }}
wc_properties_interpolate           {Input.wc.interpolate=1; }
wc_scale_gg{SPACE}+{FLOAT}          {Input.wc.scale_gg   = yytext2float(yytext,2,2); }
wc_scale_ssa{SPACE}+{FLOAT}         {Input.wc.scale_ssa  = yytext2float(yytext,2,2); }
wc_set_gg{SPACE}+{FLOAT}            {Input.wc.gg  = yytext2float(yytext,2,2); }
wc_set_ssa{SPACE}+{FLOAT}           {Input.wc.ssa = yytext2float(yytext,2,2); }
wc_set_tau{SPACE}+{FLOAT}           {Input.wc.tau = yytext2float(yytext,2,2); }
wc_set_tau550{SPACE}+{FLOAT}        {Input.wc.tau550 = yytext2float(yytext,2,2); }
wvn{FLOATS}                         {s = (char *) calloc (strlen(yytext)+1,sizeof(char));
                                     strcpy (s, yytext);
				     Input.wl.start  = yytext2float(s,2,3); 
                                     Input.wl.end    = yytext2float(yytext,3,3);
				     if (Input.wl.start > Input.wl.end) {
				       tmpfloat = Input.wl.start;
				       Input.wl.start = Input.wl.end;
				       Input.wl.end = tmpfloat;
				     }
                                     free(s);}

wvn_index{SPACE_INTEGER}+{SPACE_INTEGER}  {s = (char *) calloc (strlen(yytext)+1,sizeof(char));
                                     strcpy (s, yytext);
				     Input.wl.start_index = yytext2int(s,2,3); 
                                     Input.wl.end_index   = yytext2int(yytext,3,3);
				     if (Input.wl.start_index > Input.wl.end_index) {
				       tmpint = Input.wl.start_index;
				       Input.wl.start_index = Input.wl.end_index;
				       Input.wl.end_index = tmpint;
				     }
                                     free(s);}


zout{SIGNED_FLOATS}                 {s = (char *) calloc (strlen(yytext)+1,sizeof(char));
				     strcpy (s, yytext);
				     Input.atm.nzout    = yytext2ntokens(s); 
                                     Input.atm.zout_sur     = 
				       yytext2floats(yytext, Input.atm.nzout); 
				     free(s); }



{SPACE}+       ;         /* eat up blanks */ 

{COMMENT}*     ;         /* eat up comments */

\n                        { ++line_number;};

{WORD}                    {fprintf(stderr,"Unknown command on line %d\n", line_number+1);
                           fprintf(stderr,"%s\n", yytext);
                           ierror++;}


<include>[ \t]*      /* eat the whitespace */
<include>[^ \t\n]+   { /* got the include file name */
        if (include_stack_ptr >= MAX_INCLUDE_DEPTH) {
            fprintf (stderr, "Includes nested too deeply");
            exit (1);
        }

        include_stack[include_stack_ptr++] = YY_CURRENT_BUFFER;

        yyin = fopen( yytext, "r" );

        if (!yyin) {
            fprintf (stderr, "Error including %s\n", yytext);
            exit(1);
        }

        yy_switch_to_buffer (yy_create_buffer (yyin, YY_BUF_SIZE ));

        BEGIN(INITIAL);
        }

<<EOF>> {
        if (--include_stack_ptr < 0) {
            yyterminate();
        }
        else {
            yy_delete_buffer (YY_CURRENT_BUFFER);
            yy_switch_to_buffer (include_stack[include_stack_ptr]);
        }
        }

%%

/*************************************************************/
/* print usage information                                   */
/*************************************************************/

static void version()
{
  fprintf (stderr, "uvspec, version %s\n", VERSION);
}

static void usage()
{

  fprintf (stderr, "\n");  
  version();
  fprintf (stderr, "\nCalculates the 1D radiation field of the Earth\'s atmosphere.\n");
  fprintf (stderr, "See libRadtran documentation for more information.\n\n");
  fprintf (stderr, "No error checking yet, so be careful.\n\n");

  fprintf (stderr, "Usage: uvspec [-h] < input_file > Output_file\n");
  fprintf (stderr, " -h    Print this message.\n");  
  fprintf (stderr, "\n");  
}


int main(int argc, char **argv) 
{
  

/* @11c@

@include mystic.sdoc

@code{uvspec} calculates the radiation field in the Earth's atmosphere
for a given set of input parameters. It reads input from standard input, 
and outputs to standard output. It is normally invoked in the following
way: 

@example
@cartouche
uvspec < input_file > output_file
@end cartouche
@end example

@noindent The format of the input and output files are described
below. Several realistic examples of input files are subsequently 
given.

@noindent uvspec may produce a wealth of diagnostic messages 
and warnings, depending on your use of @code{verbose} or @code{quiet}.
Diagnostics, error messages, and warnings are written to @code{stderr} while
the uvspec output is written to @code{stdout}. To make use of this extra 
information, you may want to write the standard uvspec output to one file 
and the diagnostic messages to another. To do so, 
try @code{(./uvspec < uvspec.inp > uvspec.out) >& verbose.txt}. The 
irradiances and radiances will be written to @file{uvspec.out} while all 
diagnostic messages go into @file{verbose.txt}. This method can also be used  
to collect uvspec error messages. 


@strong{Warning:} Please note the error checking on input variables 
is not complete at the moment. Hence, if you provide erroneous input, 
the outcome is unpredictable.
  @c11@ */

  /*  @11_03c@                
@cindex input file, uvspec
@cindex uvspec, input file

The uvspec input file consists of single line entries, each making
up a complete input to the uvspec program. First on the line comes
the parameter name, followed by one or more parameter values. The 
parameter name and the parameter values are separated by white space.
Filenames are entered without any surrounding single or double quotes.
Comments are introduced by a @code{#}. Blank lines are ignored. 
The order of the lines is not important, with one exception: if the
same input option is used more than once, the second one will usually
over-write the first one.

The various input parameters are described in detail below.
@vtable @code
@item aerosol_angstrom
Scale the aerosol optical depth using the @AA{}ngstr@"om formula. Specify 
the @AA{}ngstr@"om alpha and beta coefficients. The optical thickness defined here 
is the integral from the user-definded @code{altitude} to TOA (top of atmosphere).
@item aerosol_default
Set up a default aerosol according to Shettle (1989). The default properties are 
a rural type aerosol in the boundary layer, background aerosol above 2km,
spring-summer conditions and a visibility of 50km. These settings may
be modified with @code{aerosol_haze}, @code{aerosol_vulcan}, @code{aerosol_season},
and @code{aerosol_visibility}.
@item aerosol_files
A way to specify aerosol optical depth, single scattering albedo, 
and phase function moments for each layer. The file specified by 
@code{aerosol_files} has two columns where column 1 is the altitude in km. The altitudes 
must be the same as those specified in the @code{atmosphere_file}. The second
column is a the name of a file which defines the optical properties of the layer
starting at the given altitude. The files specified in the second column must have the 
following format:
@table @code
@item Column 1: 
The wavelength in nm. These wavelengths may be different from those in @code{solar_file}. 
Optical properties are interpolated to the requested wavelengths.
@item Column 2:
The extinction coefficient of the layer in units km-1. 
@item Column 3:
The aerosol single scattering albedo of the layer.
@item Column 4-(nmom+4):
The moments of the aerosol phase function. 
@end table
For some simple examples see the files @file{examples/AERO_*.LAYER}. Note that 
if using the @code{rte_solver disort2} it makes good sense to make the 
number of moments larger than @code{nstr}. For @code{rte_solver disort} and 
@code{rte_solver polradtran} the number of moments included in the calculations 
will be @code{nstr}+1. Higher order moments will be ignored for these solvers.
@item aerosol_gg_file
Location of aerosol asymmetry parameter file. The file must have two columns.
Column 1 is the altitude in km. The altitude grid must be exactly equal to 
the altitude grid specified in the file @code{atmosphere_file}.
Column 2 is the asymmetry parameter of each layer. 
At present, the asymmetry parameter defined here is constant with wavelength
but it is planned to introduce the wavelength dependence in the near future.
Comments start with @code{#}. Empty lines are ignored.
@item aerosol_haze
Aerosol type in the lower 2 km of the atmosphere. Integer.
See E.P. Shettle, "Models of aerosols, clouds and precipitation for 
atmospheric propagation studies", in "Atmospheric propagation in the 
uv, visible, ir and mm-region and related system aspects", 
AGARD Conference Proceedings (454), 1989.

@table @code
@item 1
Rural type aerosols.
@item 4
Maritime type aerosols.
@item 5
Urban type aerosols.
@item 6
Tropospheric type aerosols.
@end table
@item aerosol_moments_file
Location of aerosol moments file, a one-column file containing an
arbitrary number of Legendre terms of the phase function. The phase function 
@iftex
 @tex
 $p(\mu)$ is
  $$ p (\mu) = \sum_{m=0}^{\infty} (2m+1) \cdot k_m \cdot P_m (\mu) $$
  where $k_m$ is the m'th moment and $P_m (\mu)$ is the m'th Legendre polynomial.
 @end tex
@end iftex
@ifinfo
 p(mu) is @*
 p (mu) = sum (m=0 to infinity) (2m+1) * k(m) * Pm (mu) @*
 where k(m) is the m'th moment and Pm (mu) is the m'th Legendre polynomial.
@end ifinfo
If not specified, a Henyey-Greenstein phase function is assumed 
where the asymmetry parameter g is either a default value depending on the 
aerosol type or may be specified using @code{aerosol_set_gg}.
@item aerosol_refrac_file
File containing the wavelength-dependent refractive index of the
aerosol. Three columns are expected: wavelength [nm] and the 
real and imaginary parts of the refractive index.
Together with @code{aerosol_sizedist_file} this forms the 
input to Mie calculations of the aerosol optical properties. Attention: 
If the aerosol properties are defined using the refractive index and the 
size distribution, the wavelength dependence of the optical properties is 
determined by Mie theory. At present there are three ways to define the 
absolute value of the optical thickness: (1) @code{visibility} defines the 
profile at the first @emph{internal} wavelength; for a monochromatic calculation
and in correlated-k mode, the first @emph{internal} wavelength equals the 
first wavelength output by @code{uvspec}; for spectral calculations, the 
first wavelength might be a little bit smaller than the first wavelength 
output by @code{uvspec}; (2) @code{aerosol_tau_file} defines the optical 
thickness profile at the first @emph{internal} wavelength; or (3) absolute
optical thickness and wavelength-dependence are defined by @code{aerosol_angstrom}.
In future it is planned to also allow the specification of 
absolute particle densities, etc.
@item aerosol_refrac_index
Wavelength-independent refractive index of the aerosol; if wavelength-dependence 
is required, use @code{aerosol_refrac_file} instead. Together with 
@code{aerosol_sizedist_file} this forms the input to Mie calculations of the 
aerosol optical properties. See comment at @code{aerosol_refrac_file}.
@item aerosol_scale_ssa
Scale the aerosol single scattering albedo for all wavelengths and altitudes
with a positive number. If the resulting scaled single scattering
albedo is larger than 1 it is set to 1.
@item aerosol_scale_tau
Scale the aerosol extinction for all wavelengths and altitudes
with a positive number.
@item aerosol_set_gg
Set the aerosol asymmetry parameter for all wavelengths and altitudes
to a constant value between -1.0 and 1.0.
@item aerosol_set_ssa
Set the aerosol single scattering albedo for all wavelengths and altitudes
to a constant value between 0.0 and 1.0.
@item aerosol_set_tau
Set the aerosol optical thickness for all wavelengths and altitudes
to a constant value. The optical thickness defined here is the
integral from the user-definded @code{altitude} to TOA (top of atmosphere). 
@item aerosol_set_tau550
Set the aerosol optical thickness at 550nm. Other wavelengths are scaled accordingly.
Note that this option requires for technical reasons that the
wavelength interval defined by @code{wvn} does contain 550nm.
The optical thickness defined here is the integral from the user-definded 
@code{altitude} to TOA (top of atmosphere). 
@item aerosol_season
Specify season to get appropriate aerosol profile.
@table @code
@item 1
Spring-summer profile.
@item 2
Fall-winter profile.
@end table
@item aerosol_sizedist_file
Aerosol size distribution. Two columns are expected: The radius
[micron] and the particle number. Together with @code{aerosol_refrac_index} or
@code{aerosol_refrac_file} this forms the input to Mie calculations of the 
aerosol optical properties. See comment at @code{aerosol_refrac_file}.
@item aerosol_ssa_file
Location of aerosol single scattering albedo file. The file must have two columns.
Column 1 is the altitude in km. The altitude grid must be exactly equal to 
the altitude grid specified in the file @code{atmosphere_file}.
Column 2 is the single scattering albedo of each layer. 
At present, the single scattering albedo defined here is constant with wavelength
but it is planned to introduce the wavelength dependence in the near future.
Comments start with @code{#}. Empty lines are ignored.
@item aerosol_tau_file
Location of aerosol optical depth file. The file must have two columns.
Column 1 is the altitude in km. The altitude grid must be exactly equal to 
the altitude grid specified in the file @code{atmosphere_file}.
Column 2 is the aerosol optical depth of each layer. 
To get wavelength dependence use the parameter @code{aerosol_angstrom}. 
Comments start with @code{#}. Empty lines are ignored.
@item aerosol_visibility
Visibility in km.
@item aerosol_vulcan
Aerosol situation above 2 km. Integer.
@table @code
@item 1
Background aerosols.
@item 2
Moderate vulcanic aerosols.
@item 3
High vulcanic aerosols.
@item 4
Extreme vulcanic aerosols.
@end table
@item albedo
The Lambertian surface albedo, a number between 0.0 and 1.0,
constant for all wavelengths. For wavelength dependent surface albedo 
use @code{albedo_file}.
@item albedo_file
Location of surface albedo file for wavelength dependent surface albedo. 
The file must have two columns.
Column 1 is the wavelength, in nm, and column 2 the corresponding 
Lambertian surface albedo. The wavelength grid may be freely set. The 
albedo will be interpolated to the wavelength grid used for the radiation
calculation. Comments start with @code{#}. Empty lines are ignored.
@item angstrom
Still supported but obsolete. Replaced by @code{aerosol_angstrom}.
@item altitude
Set the bottom level in the model atmosphere provided in 
@code{atmosphere_file} to be at altitude (km).
@example
altitude 0.73   # Altitude of IFU, Garmisch-Partenkirchen 
                # Be aware, for this to work the atmosphere
                # file must start at 0 km.
@end example
The profiles of pressure, temperature, molecular absorbers, 
ice and water clouds are cut at the specified altitude. 
The aerosol profile is not affected by @code{altitude} but starts
right from the model surface. This is a convenient way for the user to calculate the 
radiation at other altitudes than sealevel. Note that @code{altitude} is very different
from @code{zout} where the radiation is calculated at an altitude of zout
over the surface. E.g. to calculate the radiation field 1 km above the surface 
at 0.73 above sealevel, one would specify '@code{altitude} 0.73' 
and '@code{zout} 1.0'.

A second optional argument may be given to @code{altitude} as e.g.
@example
altitude 0.73 0.5
@end example
Here the bottom level will be at 0.73 km and the vertical resolution of the model
atmosphere will be redistributed to have a spacing between levels specified by the second number,
here 0.5 km. Beaware that specifying a fine vertical spacing will produce many layers thus increasing the 
computing time. Also the radiative transfer equation solvers implemented in Fortran 77 might need
to have some array sizes increased (see @file{src_f/DISORT.MXD}).
@item atmosphere_file
Location of the atmospheric data file. Must have at least three columns containing the altitude
pressure and temperature. Other traces gases are set by @code{dens_file}. 
Alternatively the first five columns must be specified and optionally columns 6 - 9.
may be omitted. Missing profiles are filled with zero's.
The up to nine columns of the file are interpreted as follows (first three columns must be specified):
@table @code
@item 1. z(km)
Altitude above sea level in km
@item 2. p(mb)
Pressure in mbar
@item 3. T(K)
Temperature in K
@item 4. air_density(cm-3) 
Air density in cm-3
@item 5. o3_density(cm-3)
Ozone density in cm-3
@item 6. o2(cm-3)
Oxygen density in cm-3
@item 7. h2o(cm-3) 
Water vapour density in cm-3
@item 8. co2(cm-3)
CO2 density in cm-3
@item 9. no2(cm-3)
NO2 density in cm-3
@end table
The atmosphere is specified
top-down, that is, the top level is the first line in the file, the bottom
(surface) level the last line. All properties refer to model @emph{level} z,
not to model @emph{layer}. It is important that the correct units are 
used, otherwise unpredictable results are guaranteed.
Comments start with @code{#}. Empty lines are ignored. libRadtran
provides the six standard atmospheres by Anderson et al. (1986):
@table @code
@item afglt
Tropical                                              
@item afglms
Midlatitude Summer                                    
@item afglmw
Midlatitude Winter                                    
@item afglss
Subarctic Summer                                      
@item afglsw
Subarctic Winter                                      
@item afglus
U.S. Standard                                         
@end table
@item brightness
Convert radiances / irradiances to brightness temperatures.
@item ch4_mixing_ratio
The mixing ratio of CH4 in ppm (default: 1.6 ppm).
@item co2_mixing_ratio
The mixing ratio of CO2 in ppm; scale the profile so that the mixing 
ratio at the user-define @code{altitude} assumes the specified value.
@item correlated_k
To calculate integrated shortwave or longwave irradiance, or to simulate 
satellite instrument channels, choose between the following 
correlated-k schemes:
@table @code
@item Kato 
Kato et al. (1999), shortwave; based on HITRAN 96. Please note that the 
bands above 2.5 micron are not very reliable which, however, has only little impact
on integrated shortwave radiation.
@item Kato2
Kato et al. (1999), shortwave; optimized version (Seiji Kato, personal communication, 2003);
please note that @code{Kato2} only has 148 subbands (that is, calls to the @code{rte_solver}) 
compared to 575 for @code{Kato} which translates to an increase in computational speed by 
up to a factor of 4 with only little increase in uncertainty. The absorption data are
based on HITRAN 2000. Please note that the bands above 2.5 micron are not very reliable which, 
however, has only little impact on integrated shortwave radiation.
@item Kato2.96
Kato et al. (1999), shortwave; optimized version (Seiji Kato, personal communication, 2003);
similar to @code{Kato2} but based on HITRAN96. Please note that the bands above 2.5 micron 
are not very reliable which, however, has only little impact on integrated shortwave radiation.
@item Fu
Fu and Liou (1992/93), shortwave and longwave; fast parameterization, developed for climate models.
@item AVHRR_KRATZ
Kratz (1995), AVHRR instrument channels
@item SBDART
Gas absorption parameterization from SBDART (Ricchiazzi et al., 1998); 
please see the section on "Spectral resolution".
@end table
If correlated_k KATO/KATO2/FU/AVHRR_KRATZ is specified, the extraterrestrial flux is taken from
internally defined files specific for each parameterization, not 
from @code{solar_file}. The output is the integrated irradiance for 
each band. To get e.g. integrated shortwave irradiance, simply add all 
bands of the Kato et al. (1999) or the Fu and Liou (1992/93)
parameterization. The five AVHRR channels are weighted sums of the
libRadtran output. Examples how to integrate the output in the
@code{AVHRR_KRATZ} case are included in the @code{uvspec} self-test
which is initiated with  
@code{make check}.
@item cox_and_munk_pcl
Pigment concentration for Cox and Munk ocean surface BRDF (in mg/m-3); 
at present only available with @code{rte_solver DISORT2}. 
The number of streams (@code{nstr}) is automatically increased
to 16 if cox_and_munk BRDF is switched on, to avoid numerical
problems. 
@item cox_and_munk_sal
Salinity for Cox and Munk ocean surface BRDF (in ppt); 
at present only available with @code{rte_solver DISORT2}.
The number of streams (@code{nstr}) is automatically increased
to 16 if cox_and_munk BRDF is switched on, to avoid numerical
problems. 
@item cox_and_munk_u10
Wind speed for Cox and Munk ocean surface BRDF (in m/s); 
at present only available with @code{rte_solver DISORT2}.
The minimum allowed wind speed is 1 m/s because otherwise the
strong specular reflection causes numerical problems. If a lower value
is specified, the wind speed is automatically set to 1m/s. Also, the
number of streams (@code{nstr}) is automatically increased to 16 if
cox_and_munk BRDF is switched on, to avoid numerical problems.
@item crs_file
May be used to specify cross sections of ozone (O3), nitrogendioxide (NO2), 
bromine oxide (BRO), OCLO, or HCHO to be used instead of one of those coming with 
libRadtran. No temperature dependence may be specified. Use as follows:
@example
crs_file NO2 ../examples/no2_crs.dat
@end example
The @code{NO2} or @code{O3,BRO} or @code{OCLO} or @code{HCHO} must be specified to identify the
specie for which the cross section applies.
The cross section file has two columns:
@table @code
@item 1. wavelength (nm)
@item 2. cross section (cm+2)
@end table
@item data_files_path
The path to the directory where all uvspec internal data files live, e.g.
the files that are in the subdirectories of the @file{data} directory
that comes with the uvspec distribution.
@item day_of_year
Integer, to correct the calculated radiation quantities for the 
Sun-Earth distance for the specified Julian day (1-365). If not given, 
the Earth-Sun distance is 1 AU (i.e. equinox distances), 
that is, no correction is applied to the extraterrestrial 
irradiance @code{solar_file}.
@item deltam
Turn delta-M scaling on/off. Set to either @code{on} or @code{off}. Note 
that for the @code{rte_solver disort2} delta-M scaling is hardcoded to 
be always on.
@item dens_column
Set the total column of a density profile. The column is integrated between the 
user-defined altitude and TOA (top of atmosphere). The syntax is
@example
dens_column specie column [unit]
@end example
where specie is on of O3, NO2, H2O, BRO, OCLO, HCHO and CO2, see also @code{dens_file}. Column is the total 
column value of the trace gas and the column is in unit which is optional. The default units are
O3 (DU), NO2 (CM_2), H2O (CM_2), BRO (CM_2), OCLO (CM_2), HCHO (CM_2) and CO (CM_2).  Here DU 
is Dobson units and CM_2 =cm-2. In addition DU may be used for NO2 as well, e.g.
@example
dens_column NO2 1.2 DU
@end example
@item dens_file
Specify density profiles (or matrix, see below) of various traces gases to be included in the radiative 
transfer calculation. At the moment ozone (O3), nitrogendioxide (NO2), water vapor (H2O),
bromine oxide (BRO), OCLO, HCHO, and carbondioxide (CO2) are included. The various density 
profiles are idenfied by their abbrevations given in the parenthesis above.
@example
dens_file O3 ../examples/afglus_o3.dat
@end example
The density file has two columns:
@table @code
@item 1. z(km)
Altitude above sea level in km, must be the same as the altitudes in @code{atmosphere_file}.
@item 2. density of trace gas (cm-3)
The density of the trace gase
@end table
To scale the profile to a total column value use @code{dens_column}.

For airmass factor calculations it is for some species neccesary to account for the 
variation of the profile with sza. This may be accomplished by using @code{dens_file} 
and feeding it a file containing the following:
@table @code
@item 1. row 1, column 1: a zero, that is 0.0
@item 2. column 1 below row 1: Altitude above sea level in km, must be the same as 
         the altitudes in @code{atmosphere_file}.
@item 3. row one after column 1: The solar zenith angle in degrees
@item 4. The rest of the matrix: the density of trace gases as a function of solar
         zenith angle and altitude.
@end table
The matrix may only be specified for one specie. It may however be combined with profiles
of other species. For examples see the @code{examples} directory.
A density matrix may only be used in connection with the @code{rte_solver sdisort}.

@item f11_mixing_ratio
The mixing ratio of F11 in ppm (default: 0.000268 ppm).
@item f12_mixing_ratio
The mixing ratio of F12 in ppm (default: 0.000503 ppm).
@item f22_mixing_ratio
The mixing ratio of F22 in ppm (default: 0.000105 ppm).
@item filter_function_file
If specified, the calculated spectrum is multiplied with a filter function 
defined in @file{filter_function_file}. The file must contain two columns. 
Column 1 is the wavelength, in nm. Column 2 is the corresponding filter 
function value. Comments start with @code{#}. Empty lines are ignored.
In combination with @code{output sum} or @code{output integrate} this option 
is useful e.g. to calculate weighted irradiances or actinic fluxes or 
to simulate broadband or satellite observations.
@item fisot
Specifies that isotropic illumination is used at top-boundary instead
of beam source. Useful for those who want to calculate the reflectance 
for a homogeneous or inhomogeneous atmosphere. The intensity is still set by
@code{solar_file}.
@item flexstor
Output is in flexstor format. May not be combined with @code{header}.
Also, does not currently work when @code{umu} and/or @code{phi} is specified.
@item h2o_mixing_ratio
The mixing ratio of H2O in ppm. Scale the profile so that the mixing 
ratio at the user-define @code{altitude} assumes the specified value.
@item h2o_precip
Precipitable water in kg / m2 (which is approximately 1mm). 
The water vapor profile is scaled accordingly. The precipitable water 
is integrated from the user-defined @code{altitude} to TOA (top of atmosphere).
@item header
Output information on some of the input parameters. May not be combined
with @code{flexstor}.
@item n2o_mixing_ratio
The mixing ratio of N2O in ppm (default: 0.28 ppm).
@item ic_file
Location of file defining ice cloud properties. The file must
contain three columns. Column 1 is the altitude in km, column 2 
the ice water content in grams per qubic meter, and column 3 the effective
particle radius in micron. The ice water content and effectice particle
radius must be specified at the same altitude grid as 
in @code{atmosphere_file}. Note that the definition of cloud altitudes   
in @code{ic_file} refers to sea level, not to altitude above
ground. E.g., when altitude is set to 1.63km, and the first cloud level is 
defined at 3km, the cloud would start at 1.37km above ground.
The optical depth of a layer is calculated 
using information from the upper and lower levels defining the layer. 
Comments start with @code{#}. Empty lines are ignored. An example 
of an ice cloud is given in @file{examples/IC.DAT}. Per default, the cloud properties are 
interpreted as properties at a given altitude @emph{level}. If @code{ic_layer} is defined,
they are interpreted as @emph{layer} properties.
@item ic_files
A way to specify ice cloud optical depth, single scattering albedo, 
and phase function moments for each layer. The file specified by 
@code{ic_files} has two columns where column 1 is the altitude in km. The altitudes 
must be the same as those specified in the file @code{atmosphere_file}. The second
column is the name of a file which defines the optical properties of the level 
starting at the given altitude. The files specified in the second column must 
have the following format:
@table @code
@item Column 1: 
The wavelength in nm. These wavelengths may be different from those in @code{solar_file}. 
Optical properties are interpolated to the requested wavelengths.
@item Column 2:
The extinction coefficient of the layer in units km-1. 
@item Column 3:
The single scattering albedo of the layer.
@item Column 4-(nmom+4):
The moments of the scattering phase function. 
@end table
Note that if using the @code{rte_solver disort2} it makes good sense to make the 
number of moments larger than @code{nstr}. For @code{rte_solver disort} and 
@code{rte_solver polradtran} the number of moments included in the calculations 
will be @code{nstr}+1. Higher order moments will be ignored for these solvers.
@item ic_habit
Ice crystal habit for the Yang et al. (2000), Key et al. (2002)
parameterization, see also  @code{ic_properties yang}. May be one of solid-column, 
hollow-column, rough-aggregate, rosette-4, 
rosette-6, or plate.
@item ic_ipa_files
A two-column file, defining ice cloud property files (see @code{ic_file}) in the first 
column and the correspoding weights in the second column. The radiative transfer 
calculation is performed independently for each cloud column and the result is 
the weighted average of all independent columns. If @code{ic_ipa_files} and 
@code{wc_ipa_files} are both defined, both must have the same columns in the same 
order, otherwise @code{uvspec} will complain.
@item ic_layer
Specify ice cloud properties for model @emph{layers} instead of @emph{levels} 
(see also @code{ic_file}). 
@item ic_properties
Defines how ice water content and effective particle radius are translated 
to optical properties. Possible choices are
@table @code
@item fu 
Parameterization by Fu (1996), Fu et al. (1998), see ic_file; 
this is the default setting. Note that this is a parameterization
which has been created to calculate fluxes but not radiances. 
Note also that the optical properties in the solar range provided by 
Fu (1996) are delta-scaled properties (that is, the forward peak of
the phase function is truncated and optical thickness, asymmetry
parameter, and single scattering albedo are reduced accordingly).
For wavelengths up to 4 micron Fu (1996) is used while for wavelengths 
larger than 4 micron Fu et al. (1998) is chosen. Please note that 
Fu (1996) is based on ray-tracing calculations while Fu et al. (1998)
is a mixture of ray-tracing and Mie calculations (which is required for 
the infrared wavelengths where the geometrical assumption does not hold).
Hence, both parameterizations are not fully consistent. Rather, differences
of some % are to be expected in the wavelength region where both 
parameterizations overlap.
@item yang
Parameterization by Yang et al. (2000), Key et al. (2002). This
parameterization can also be used to calculate radiances because it 
uses a double-Henyey-Greenstein phase function which better represents
both forward and backward peaks. This parameterization covers the region 
from 0.2 to 4.8 micron. It has been complemented by the parameterization 
by Yang et al. (2001) which covers the wavelength region 8 to 13 micron.
@item mie
Use pre-calculated Mie tables; useful for @code{correlated_k}; 
the tables are expected in @code{data_files_path}/correlated_k/../ @*
For spectral or pseudo-spectral (@code{correlated_k sbdart}) calculations, 
a set of pre-calculated tables is also available; the wavelength grid points of these 
data has been carefully selected such that the extinction cross section, 
single scattering albedo, and the asymmetry parameter are accurate to 1% 
(compared to the fully-resolved Mie calculation) for all wavelengths
between 250nm and 100 micron. For spectral or pseudo-spectral
calculations @code{wc_properties_interpolate} has to be defined explicitely to
initiate the interpolation of the optical properties to the internal wavelength grid.
Please note that this option may be extremely memory-comsuming because for each 
internal wavelength a full set of Legendre moments of the phase function is 
stored (up to several thousands). The Mie tables are not part of the standard distribution 
(because of their large size) but they are freely available from http://www.libradtran.org. 
Note that a Mie calculation assumes spherical ice particles, the scattering function of 
which differs systematically from non-spherical particles. Hence, @code{ic_properties mie}
is usually not representative of natural ice clouds.
@item filename
Read optical properties from specified filename; file format is as produced 
by Frank Evans' @code{cloudprp}; for each of the internal (computational) wavelengths, 
a separate file is expected (this option is subject to change).
@end table
Please note also that, in contrast to spherical particles, there is no unique 
definition of effective size for non-spherical particles. In particular, the
above parameterizations use different definitions which, however, differ only by 
a constant factor. 
@iftex
 @tex
  Yang et al. (2000), Key et al. (2002) use the general definition 
  $$ r_{\rm eff} = {{3}\over{4}}{{\int V(h) n(h) dh}\over{\int A(h) n(h) dh}}$$
  where h is the maximum dimension of an ice crystal, n(h) is the
  number of particles with maximum dimension h in the size distribution,
  and V and A are the volume and mean
  projected area of the particles, respectively. The volume and area are
  based on the spherical diameter with equivalent volume and the
  spherical diameter with equivalent projected area as defined by 
  Yang et al. (2000). On the other hand, Fu et al. [1996,1998] use
  hexagonal columns and use the following definition
  $$ r_{\rm eff} =  {{\int D^2 L n(L) dL}\over{2 \int (D L + {\sqrt{3}\over{4}} D^2) n(L) dL}}$$
  where D is the width of the ice crystal (that is, the maximum diameter of the 
  hexagonal area) and L is the length. The integrand in the numerator is proportional 
  to the volume while that in the denominator is proportional to the projected area. 
  Evaluating these formulas one finds that, for the same hexagonal particle, the effective
  radius would be $3 \sqrt{3} / 4 = 1.299$ times larger following the 
  Yang et al. (2000), Key et al. (2002) definition than the Fu (1996,1998) definition.
  As an example, an effective radius of 20$\mu m$ with "ic\_properties fu" and
  1.299 $\cdot$ 20$\mu m$ = 26$\mu m$ with "ic\_properties yang" would give comparable results
  for hexagonal columns. 
 @end tex
@end iftex
@ifinfo
@end ifinfo
@item ic_properties_interpolate
Interpolate ice cloud optical properties over wavelength; useful for precalculated 
optical property files defined with @code{ic_properties}. Please note that this option 
may be extremely memory-comsuming because for each internal wavelength a full set of 
Legendre moments of the phase function is stored (up to several thousands). 
@item ic_scale_gg
Scale the ice cloud asymmetry factor for all wavelengths and altitudes
with a float between 0.0 and 1.0.
@item ic_scale_ssa
Scale the ice cloud single scattering albedo for all wavelengths and altitudes
with a float between 0.0 and 1.0.
@item ic_set_gg
Set the ice cloud asymmetry factor for all wavelengths and altitudes
to a float between -1.0 and 1.0.
@item ic_set_ssa
Set the ice cloud single scattering albedo for all wavelengths
and altitudes to a value between 0.0 and 1.0.
@item ic_set_tau
Set the total ice cloud optical depth to a constant value for all wavelengths.
The optical thickness defined here is the integral from the surface at the 
user-defined @code{altitude} to TOA (top of atmosphere). This option is useful 
only for monochromatic calculations or in wavelength regions 
where the optical properties of ice clouds can be considered constant, e.g. the 
ultraviolet region.
@item ic_set_tau550
Set the ice cloud optical thickness at 550nm. Other wavelengths are scaled accordingly.
The optical thickness defined here is the integral from the surface at the 
user-defined @code{altitude} to TOA (top of atmosphere). 
Note that this option requires for technical reasons that the
wavelength interval defined by @code{wvn} does contain 550nm.
@item include
Include a file into the uvspec input. Works exactly like the C @code{#include} or 
the Fortran @code{INCLUDE} statements.
@ifset mystic
@item mc_absorption
Calculate MYSTIC absorption of each grid box; may need a considerable amount of memory,
depending on the 3D cloud grid. Only meaningful with @code{rte_solver montecarlo}.
@item mc_actinic
Calculate MYSTIC actinic flux for each grid box by dividing the absorbed energy by the 
absorption coeffcient; this method is much better than the traditional photon counting 
which usually comprises spikes (because in the latter method each photon is weighted with 
1/cos(theta) which may be a very large number); may need a considerable amount of memory, 
depending on the 3D cloud grid. Only meaningful with @code{rte_solver montecarlo}. 
@item mc_basename
Filename for MYSTIC 3D output (default: mc).
@item mc_albedo_file
Define a MYSTIC 2D albedo input file (default: albedo2D.dat).
@item mc_cloud_grid
3D cloud grid size (Nx,Ny,Nz) for MYSTIC. Weird, but required to keep MYSTIC 
a function that can be called from a Fortran program.
Only meaningful with @code{rte_solver montecarlo}.
@item mc_elevation_file
Define a MYSTIC 2D elevation input file (default: elevation2D.dat).
@item mc_emission
Calculate MYSTIC emission of each grid box; may need a considerable amount of memory,
depending on the 3D cloud grid. Note that emission is calculated directly without tracing 
any photons. Increasing @code{mc_photons} will therefore not change the result.
Only meaningful with @code{rte_solver montecarlo} and @code{source thermal}. 
@item mc_escape
Calculate MYSTIC radiances via escape probabilities; slows down the tracing 
but usually requires much less photons. Only meaningful with @code{rte_solver montecarlo}.
@item mc_heating
Calculate MYSTIC heating rates for each grid box; may need a considerable amount of memory,
depending on the 3D cloud grid. Note that emission is calculated directly without tracing 
any photons. Increasing @code{mc_photons} will therefore not change the result.
Only meaningful with @code{rte_solver montecarlo} and @code{source thermal}. 
@item mc_ipa
Run MYSTIC in independent pixel mode. Only meaningful with @code{rte_solver montecarlo}.
@item mc_momcld_file
Cloud phase function moments file for MYSTIC; can be any filename or 
'water' or 'ice'. In the latter case, pre-calculated tables for water 
and ice are used. For each of the internal (computational) wavelengths, 
a separate file is expected.
Only meaningful with @code{rte_solver montecarlo}.
@item mc_photons
Total number of photons to be traced by the Monte Carlo solver, MYSTIC.
Only meaningful with @code{rte_solver montecarlo}.
@item mc_pcld_file
Cloud phase function file for MYSTIC.
Only meaningful with @code{rte_solver montecarlo}.
@item mc_photons
Total number of photons to be traced by the Monte Carlo solver, MYSTIC.
Only meaningful with @code{rte_solver montecarlo}.
@item mc_photons_file
Distribution of photons over wavelength bands; to be used with @code{correlated_k};
for an example see @file{data/correlated_k/kato/photons.dat}. No error checking yet!
Do only use if you are absolutely sure what you are doing.
Only meaningful with @code{rte_solver montecarlo}.
@item mc_rpv_file
Define a MYSTIC 2D RPV BRDF input file.
@item mc_rpv_type
File containing wavelength-dependent RPVs to be used in combination with @code{mc_rpv_file}.
@item mc_sample_grid
Sampling grid size (Nx,Ny) for MYSTIC.
Only meaningful with @code{rte_solver montecarlo}.
@item mc_truncate
Truncate phase function at the specified polar angle mu. 
USE ONLY IF YOU REALLY KNOW WHAT YOU ARE DOING!
@item mc_wcloud_file
Define a MYSTIC 3D cloud input file (default: wcloud3D.dat).
@end ifset
@item molecular_tau_file
Location of molecular absorption optical depth file. Usually, molecular absorption
is calculated from trace gas concentrations provided in @code{atmosphere_file}
(scaled with @code{ozone_column}, etc. Use this option only if you want to specify
the optical depth directly (e.g. for a model intercomparison) or for a line-by-line
calculation. If a spectral @code{molecular_tau_file} is specified, the wavelength 
grid defined there is used as internal wavelength grid for the radiative transfer
calculation, if not defined otherwise with @code{transmittance_wl_file}.
@code{molecular_tau_file} can be either of the following three formats:
@table @asis
@item Monochromatic:
Column 1 is the altitude in km where the altitude grid must be exactly equal 
to the altitude grid specified in @code{atmosphere_file}. 
Column 2 is the absorption optical depth of each layer.
@item Spectral, ASCII:
The first line contains the level altitudes in decreasing order; the following lines 
contain the wavelength [nm] in the first column and then the absorption optical depths
of each layer. 
@item Spectral, netcdf:
netcdf is a common platform independent format; the description, a library to read and 
write netcdf including some tools to generate netcdf is available at 
http://www.unidata.ucar.edu/packages/netcdf/. A @code{molecular_tau_file} must obey 
certain rules; an example is available at the libRadtran homepage, 
@file{UVSPEC.O2A.afglms.cdf}, a line-by-line spectrum of the oxygen A-Band 
around 760nm, calculated for the mid-latitude summer 
atmosphere by Anderson et al. [1986]. The advantage of
netcdf compared to ASCII is that it is much faster to read, and that the file 
is a self-contained, including data and a description of the variables and arrays.
It is therefore particularly useful for line-by-line calculations where usually 
many spectral data points are involved.
@end table
Comments start with @code{#}. Empty lines are ignored.
@item no_absorption
Switch absorption off.
@item no_molecular_absorption
Switch all (molecular, aerosol, cloud, and ice cloud) absorption off. Note that
this option does not affect the single scattering albedo; e.g. with an aerosol
optical depth of 1 and a single scattering albedo of 0.7, the scattering optical 
depth will still be 0.7, even with @code{no_absorption}. 
@item no_rayleigh
Switch Rayleigh scattering off.
@item no_scattering
Switch scattering off.
@item no2_column_du
Obsolete, use @code{dens_column} instead.
Set the NO2 column to a given value. The column is integrated between the 
user-defined @code{altitude} and TOA (top of atmosphere). The value must be in 
Dobson units. If value is negative or @code{no2_column_du}
is not specified, the NO2 column is not scaled.
@item no2_column_moleccm-2
Obsolete, use @code{dens_column} instead.
Set the NO2 column to a given value. The column is integrated between the 
user-defined @code{altitude} and TOA (top of atmosphere). The value must be in 
molecules / cm2. If value is negative or @code{no2_column_moleccm-2}
is not specified, the NO2 column is not scaled.
@item nscat
The order of scattering for the @code{sos} radiative transfer equation solver. 
Default is 20. May also be used with the @code{sdisort} solver. If set to 1 @code{sdisort} 
will run in single scattering mode while if set in to 2, @code{sdisort} runs in full multiple 
scattering mode.
@item nrefrac
For the @code{rte_solver sdisort} refraction may be included by specifying 
@code{nrefrac}. If refraction is included also set parameter 
@code{refraction_file}.
@table @code
@item 0
No refraction, default.
@item 1
Refraction included using fast, but harsh method.
@item 2
Refraction included using slow, but accurate method.
@end table
@item nstr
Number of streams used to solve the radiative transfer equation.
Default is 6 for fluxes and 16 for radiances.
@item o3_crs
Choose between the following ozone cross sections.
@table @code
@item Bass_and_Paur
Bass and Paur ozone cross section.
@item Molina
Molina and Molina (1986) ozone cross section.
@item Daumont
Ozone cross section by Daumont et al. (1992), Malicet et al. (1995).
@end table
Molina and Molina is default.
@item ozone_column
Obsolete, use @code{dens_column} instead.
Set the ozone column to a given value. The column is integrated between the 
user-defined @code{altitude} and TOA (top of atmosphere). The value must be in 
Dobson units. If value is negative or @code{ozone_column}
is not specified, the ozone column is not scaled.
@item output
Output processing. Choose between the following options:
@table @code
@item sum
Sum output over wavelength. Useful in combination with the 
@code{correlated_k} option.
@item integrate
Integrate output over wavelength. Useful for spectral calculations.
@item none
No processing - output spectral information (default).
@end table
@item phi
Azimuth output angles (in degrees) in increasing order. 
The radiance is output at @code{phi} and @code{umu}.
@item phi0
Azimuth angles of the sun (0 to 360 degrees). If @code{phi0} varies as a
function of wavelength use @code{sza_file}.
@item polradtran_aziorder
Order of Fourier azimuth series: 0 is azimuthially symmetric case.
Default 0.
@item polradtran_max_delta_tau
Initial layer thickness for doubling; governs accuracy, 10E-5 should be
adequate. Do not go beyond half the real precision, i.e. 10e-8 for REAL*8.
Default 1.e-05.
@item polradtran_nstokes
Number of Stokes parameters
@table @code
@item 1 
for I (no polarization)
@item 2
for I,Q
@item 3
for I,Q,U
@item 4 
for I,Q,U,V
@end table
Default is 1.
@item polradtran_quad_type
Type of quadrature used: 
@table @code
@item G
gaussian
@item D
double gaussian, 
@item L
Lobatto
@item E
extra-angle(s), this must be used of @code{polradtran} is used in 
combination with @code{umu}. Will internally use Gaussian scheme (@code{G}). 
See also radtran documentation.
@end table
Default G.                                  
@item polradtran_src_code
Radiation sources included:
@table @code
@item 0
none
@item 1
solar
@item 2
thermal
@item 3
both
@end table
Default 1.
@item pressure
The surface pressure (at the user-defined @code{altitude} in hPa. 
The pressure profile and density profiles are scaled accordingly.
@item prndis
Specify one or more integers between 1 and 7.
Print various disort input and output in disorts own format. See 
@file{disort.doc} for more information.
@strong{Warning:} Produces a lot of output. 
@item quiet
If specfied, a number of informative messages about what options are used and
progress during computations are turned off. Otherwise these messages are output 
to stderr. See also @code{verbose}.
@item rayleigh_tau_file
Location of Rayleigh scattering optical depth file. Usually, the Rayleigh scattering
cross section is calculated from the air pressure provided in @code{atmosphere_file}
(scaled with @code{pressure}. Use this option only if you really want to specify
the optical depth directly (e.g. for a model intercomparison). Note also that 
no wavelength dependence can be specified; hence this option is only useful for 
monochromatic calculations. The  @code{rayleigh_tau_file} must have two columns. 
Column 1 is the altitude in km where the altitude grid must be exactly equal 
to the altitude grid specified in @code{atmosphere_file}. 
Column 2 is the Rayleigh scattering optical depth of each layer.
Comments start with @code{#}. Empty lines are ignored.
@item reflectivity
Calculate transmission / reflectivity instead of absolute quantities. 
For irradiances / actinic fluxes the transmission T is defined as 
@iftex
 @tex
  $$ T = {{E}\over{E_0 \cos{\theta}}}$$
  where $E$ is the irradiance / actinic flux, $E_0$ is the extraterrestrial flux,
  and $\theta$ is the solar zenith angle.
 @end tex
@end iftex
@ifinfo
 T = E / (E0 cos(theta)) @*
 where E is the irradiance / actinic flux, E0 is the extraterrestrial flux,
 and theta is the solar zenith angle.
@end ifinfo
The reflectivity R is defined as 
@iftex
 @tex
  $$ R = {{\pi \cdot L}\over{E_0 \cos{\theta}}}$$
 where $L$ is the radiance, $E_0$ is the extraterrestrial flux,
 and $\theta$ is the solar zenith angle.
 @end tex
@end iftex
@ifinfo
 R = pi L / (E0 cos(theta)) @*
 where L is the radiance, E0 is the extraterrestrial flux,
 and theta is the solar zenith angle.
@end ifinfo
Obviously, reflectivities do not depend on Sun-Earth distance. Please 
note the difference to @code{transmittance}.
@item refractive_index_file
Location of refractive index file. The file must have two columns.
Column 1 is the altitude in km. The altitude grid must be exactly equal to 
the altitude grid specified in the file @code{atmosphere_file}.
Column 2 is the refractive index of each layer. 
Only has effect if used with the @code{rte_solver sdisort} and @code{nrefrac}
different from zero.
Comments start with @code{#}. Empty lines are ignored.
@item reverse
Option for the strong and bold. Reverses the atmospheric input to the radiative
transfer solvers. That is, the atmosphere is turned on the head.
Yes, that is actually useful for some purposes. If you think
you need this contact the author. Otherwise, do not use.
@item rh_file
File that defines a profile of relative humidity. If specified, the water vapour 
profile in atmosphere_file is over-written. If -1 is specified at a level, the value 
from atmosphere_file is used.
@item rpv_file
4 column file, containing the Rahman, Pinty, and Verstraete (RPV) BDRF
parameterization, Rahman et al. (1993).
Bidirectional reflectance distribution functions for a variety of surfaces are 
given in the paper. This option in only supported with DISORT 2.0 and
MYSTIC. The columns of the input file are wavelength [nm], rho0, k, and theta. 
The parameters are interpolated linearely to the internal wavelength grid. 
To make sure that the results are reasonable, specify the RPV data on a wavelength 
grid similar or equal to that used internally for the radiative transfer calculation!
@item rpv_k
Constant RPV rho0, see @code{rpv_file}. @code{rpv_k} overwrites the wavelength-dependent
value defined in @code{rpv_file}.
@item rpv_rho0 
Constant RPV rho0, see @code{rpv_file}. @code{rpv_rho0} overwrites the wavelength-dependent
value defined in @code{rpv_file}.
@item rpv_theta
Constant RPV theta, see @code{rpv_file}. @code{rpv_theta} overwrites the wavelength-dependent
value defined in @code{rpv_file}.
@item rte_solver
Set the radiative transfer equation solver to be used. Options are
@vtable @code
@item disort
The standard plane--parallel disort algorithm by Stamnes et
al. (1988), version 1.3. For documentation see @file{src_f/DISORT.doc} as well as  the
papers and the DISORT report at @*
ftp://climate.gsfc.nasa.gov/pub/wiscombe/Multiple_Scatt/. To optimize for computational time 
and memory, please adjust the parameters in src_f/DISORT.MXD for your application and re-compile. 
@item disort2
Version 2 of disort. For documentation see @file{src_f/DISORT2.doc} as well as  the
papers and the DISORT report at @*
ftp://climate.gsfc.nasa.gov/pub/wiscombe/Multiple_Scatt/.
disort2 has several improvements compared to its 'ancestor' disort
1.3. Hence we recommend to use disort2 rather than the older version.
To optimize for computational time 
and memory, please adjust the parameters in src_f/DISORT.MXD for your application and re-compile. 
@item sdisort
Pseudospherical disort as described by Dahlback and Stamnes (1991). Double precision
version. To optimize for computational time and memory, please adjust the parameters 
in src_f/DISORT.MXD for your application and re-compile. 
@item spsdisort
Pseudospherical disort as described by Dahlback and Stamnes (1991). Single precision 
version, not recommended for large solar zenith angles. 
@strong{Warning:} it is not recommended to use sdisort or spsdisort for cloudy conditions. 
With clouds included it may be numerically unstable and produce wrong results.
To optimize for computational time and memory, please adjust the parameters in 
src_f/DISORT.MXD for your application and re-compile. 
@item polradtran
The plane-parallel radiative transfer solver of Evans and Stephens (1991).
Includes polarization. Note that polarization effects of aerosols and 
clouds are currently not included.
@item twostr
The two--stream radiative transfer solver described by Kylling et al. (1995).
@item sos
A scalar pseudospherical succesive orders of scattering code. Works for 
solar zenith angles smaller than 90 degrees. Can calculate azimuthally 
averaged radiances. Set @code{nscat} to specify the order of scattering.
@item montecarlo
The MYSTIC code, see http://www.bmayer.de/mystic.html. Note that
MYSTIC is not part of the libRadtran distribution at
present. However, it has been given to some users on a collaborative basis.
@item tzs
TZS stands for "thermal, zero scattering" and is a very fast analytical solution 
for the special case of thermal emission in a non-scattering atmosphere. Please 
note that TZS does only radiance calculations at top of the atmosphere.
@item sss
SSS stands for "solar, single scattering" and is an analytical single
scattering approximation which might be reasonable for a optically
thin atmosphere. Please note that SSS does only radiance calculations 
at top of the atmosphere. 
@end vtable
@item slit_function_file
If specified, the calculated spectrum is convolved with the function found
in the @file{slit_function_file}. The file must
contain two columns. Column 1 is the wavelength, in nm, and relative to the
center wavelength. Column 2 is the corresponding slit function value. It must
be unity at the maximum. The wavelength steps in the slit function file must
be equidistant. Comments start with @code{#}. Empty lines are ignored.
@item solar_file
Location of file holding the extraterrestrial spectrum. The file must
contain two columns. Column 1 is the wavelength, in nm, and column 2 
the corresponding extraterrestrial flux. The user may freely use any units
he/she wants on the extraterrestrial flux. The wavelength grid specified
defines the wavelength resolution at which results are returned. However, 
the wavelength range is determined by @code{wvn}. @code{solar_file} may be 
omitted for thermal radiation calculations (@code{source thermal}) as well as 
@code{transmittance} and @code{reflectivity} calculations. If omitted, the 
output resolution equals the internal wavelength grid which the model choses 
for the radiative transfer calculation.
Comments start with @code{#}. Empty lines are ignored.
Note that @code{solar_file} is ignored if @code{correlated_k} is specified.
@item source
Solar or thermal source. Set to either @code{solar} or @code{thermal}.
@item spline
Spline interpolate to wavelengths lambda_0 to lambda_1 in steps of lambda_step,
in nm. Specified as e.g.
@example
spline 290. 365. 0.5
@end example
Here, the calculated spectrum is interpolated to wavelengths 290., 290.5, 291., ...
, 364.5, 365. For interpolation to arbitrary wavelengths use @code{spline_file}.
The specified wavelength interval must be within the one specified by @code{wvn}.
@item spline_file
Spline interpolate to arbitrary wavelengths, in nm, given as a single column in file 
@file{spline_file}. The specified wavelengths must be within the range specified 
by @code{wvn}. Comments start with @code{#}. Empty lines are ignored.
@item surface_temperature
Surface temperature, used for thermal infrared calculations. If not specified, the
temperature of the lowest atmospheric level is used as surface temperature.
@item sza
The solar zenith angle. If the solar zenith angle varies with wavelength,
use @code{sza_file}. 
@item sza_file
Location of solar zenith angle file for wavelength dependent solar
zenith angle. The file must have two or three columns. Column 1 is the wavelength, 
in nm, and column 2 the corresponding  solar zenith angle. Optionally
the third column may contain the corresponding solar azimuth angle. The solar 
azimuth angle is only needed when calculating radiances. The wavelength 
grid may be freely set. The solar zenith and azimuth angle will be interpolated to 
the wavelength grid used for the radiation calculation.
Comments start with @code{#}. Empty lines are ignored.
@item thermal_bands_file
File with the center wavelengths and the wavelength band intervals to be used for 
calculations in the thermal range. The following three columns are expected:
center (or reference) wavelength, lower wavelength limit, upper wavelength limit [nm].
@code{thermal_bands_file} defines the wavelength grid for the radiative transfer 
calculation. The RTE solver is called for each of the wavelengths in the first column. 
The atmospheric (scattering, absorption, etc) properties are also evaluated at these 
wavelengths. For thermal radiation calculations, the Planck function is integrated 
over the wavelength bands defined in the second and third columns. The result will
therefore be a band-integrated irradiance which does only make sense when the 
@code{solar_file} grid equals the @code{thermal_bands_file} grid.
@item thermal_bandwidth
Specify a constant bandwidth in cm-1 for thermal calculations. The default is 1 cm-1.
This option is ignored if the bands are defined explicitely, like with
@code{thermal_bands_file} or @code{correlated_k KATO/FU/AVHRR_KRATZ}.
@item transmittance
Calculate transmittance / reflectance instead of absolute quantities. 
That is, set the extraterrestrial irradiance to 1 and do not correct for Sun-Earth distance:
@iftex
 @tex
  $$ T = {{E}\over{E_0}}$$
  where $E$ is the irradiance / actinic flux / radiance and $E_0$ is the extraterrestrial flux.
 @end tex
@end iftex
@ifinfo
 T = E / E0 @*
 where E is the irradiance / actinic flux and E0 is the extraterrestrial flux.
@end ifinfo
Please note the difference to @code{reflectivity}.
@item transmittance_wl_file
Location of single column file that sets the wavelength grid used for the 
internal transmittance calculations. The wavelengths must be in nm.
Do not use this option unless you know what you are doing.
Comments start with @code{#}. Empty lines are ignored.
@item umu
Cosine of output polar angles in increasing order, starting with 
negative (downward) values (if any) and on through positive (upward)
values. Must not have any zero values. The azimuthally averaged intensity
@code{u0u} is output at @code{umu}.
@item verbose
If specified abundances of informative messages are output to stderr. To make
use of this information, you may want to write the standard uvspec output to 
one file and the diagnostic messages to another. To do so, try
@code{(./uvspec < uvspec.inp > uvspec.out) >& verbose.txt}. The irradiances 
and radiances will be written to @file{uvspec.out} while all 
diagnostic messages go into @file{verbose.txt}. See also @code{quiet}.
@item wc_cloudcover
Set the fraction of the horizontal sky area which is covered by clouds.
When a cloud cover is specified, the result will be calculated by
the independent pixel approximation (IPA), that is, as weighted average 
of cloudless sky and overcast sky, where the cloud properties are taken 
from @code{wc_file}, etc. When @code{wc_cloudcover} is specified, 
@code{wc_ipa} is switched on
automatically (see also 
@code{wc_ipa}).
@item wc_file
Location of file defining water cloud properties. The file must
contain three columns. Column 1 is the altitude in km, column 2 
the liquid water content in grams per qubic meter, and column 3 the effective
droplet radius in micron. The liquid water content and effectice droplet 
radius must be specified at the same altitude grid as 
in @code{atmosphere_file}. Note that the definition of cloud altitudes   
in @code{wc_file} refers to sea level, not to altitude above
ground. E.g., when altitude is set to 1.63km, and the first cloud level is 
defined at 3km, the cloud would start at 1.37km above ground.
The optical depth of a layer is calculated 
using information from the upper and lower levels defining the layer. 
Comments start with @code{#}. Empty lines are ignored. An example 
of a cloud is given in @file{examples/WC.DAT}. Per default, the cloud properties are 
interpreted as properties at a given altitude @emph{level}. If @code{wc_layer} is defined,
they are interpreted as @emph{layer} properties.
@item wc_files
A way to specify cloud extinction coefficient, single scattering albedo, 
and scattering phase function for each layer. The file specified by 
@code{wc_files} has two columns where column 1 is the altitude in km. The altitudes 
must be the same as those specified in the file @code{atmosphere_file}. The second
column is the name of a file which defines the optical properties of the layer 
starting at the given altitude. The files specified in the second column must 
have the following format:
@table @code
@item Column 1: 
The wavelength in nm. These wavelengths may be different from those in @code{solar_file}. 
Optical properties are interpolated to the requested wavelengths.
@item Column 2:
The extinction coefficient of the layer in units km-1. 
@item Column 3:
The single scattering albedo of the layer.
@item Column 4-(nmom+4):
The moments of the scattering phase function. 
@end table
Note that if using the @code{rte_solver disort2} it makes good sense to make the 
number of moments larger than @code{nstr}. For @code{rte_solver disort} and 
@code{rte_solver polradtran} the number of moments included in the calculations 
will be @code{nstr}+1. Higher order moments will be ignored for these solvers.
@item wc_ipa
Location of the 3D cloud field for the Independent pixel approximation (IPA).
Cloud information is read from the file. The three-dimensional cloud 
field is split into individual columns which then are considered 
one-dimensional and treated separately with the specified RTE solver.
The output quantities are averages over all columns.  The format of the file 
holding the 3D cloud field is as follows:

@example

  80 80 94 1 1
  0.025 0.025 0.0 0.5 0.75 1.0 1.4 1.5 ...
   
  # layer 3,  0.75 - 1.0 km
  1 80 4 1.25 0.86 0.99999
  2 80 4 1.25 0.86 0.99999
  3 80 4 1.25 0.86 0.99999
  4 80 4 1.25 0.86 0.99999
  ...
@end example

@enumerate    
@item 1st line: Nx  Ny  Nz  gspec wspec
 
@table @asis
@item   Nx     
number of boxes in x direction
@item   Ny     
number of boxes in y direction
@item   Nz     
number of boxes in z direction
@item   gspec  
if > 0, asymmetry parameter is specified for each cell
@item   wspec  
if > 0, single scattering albedo is specified for each cell

@end table
Grid boxes are always counted from 1 .. Nx, 1 .. Ny, 1 .. Nz.
@item 2nd line: dx  dy  z[1] z[2] z[3] ... z[Nz] z[Nz+1]
@table @asis
@item  dx  
size of boxes in x direction [km]
@item  dy  
size of boxes in y direction [km]
@item  z[1] .. z[Nz+1]   
z-grid [km]. Note that this grid must be exactly the same 
as the 1D grid in @code{atmosphere_file}
@end table
@item 3rd line, etc.   (cloud definition lines)
@example
       ix  iy  iz  ext  g  w
@end example
@table @asis
@item   ix   
index of grid box in x direction  (1 .. Nx)
@item   iy   
index of grid box in y direction  (1 .. Ny)
@item   iz   
index of grid box in z direction  (1 .. Nz)
@item   ext  
extinction coefficient [1/km]
@item   g    
asymmetry parameter (only used if gspec > 0)
@item   w    
single scattering albedo (only used if wspec > 0)
@end table
@end enumerate
 
@emph{Rules to set up the atmosphere:}

@enumerate
@item A 1D cloudless atmosphere is built by uvspec, as usual.
@item Each layer for which a cloud definition line is found in
      @code{filename} will be overwritten by 3D clouds. 
      If gspec or wspec are not set,
      asymmetry parameter or single scattering albedo will be
      taken from the 1D definition. 
@item If only properties for some boxes of a layer are defined,
      the extinction coefficients of all other boxes of this 
      layer will be set to 0.
@end enumerate

@item wc_ipa_files
A two-column file, defining water cloud property files (see @code{wc_file}) in the first 
column and the corresponding weights in the second column. The radiative transfer 
calculation is performed independently for each cloud column and the result is the weighted 
average of all independent columns. If @code{ic_ipa_files} and @code{wc_ipa_files} 
are both defined, both must have the same columns in the same order, 
otherwise @code{uvspec} will complain.
@item wc_layer
Specify cloud properties for layers instead of levels (see also 
wc_file). If wc_layer is specified, cloud properties are 
assumed to be constant over the layer.
@item wc_properties
Define how liquid water content and effective droplet radius are translated 
to optical properties. Possible choices are
@table @code
@item hu 
Parameterization by Hu and Stamnes (1993); 
this is the default setting. Note that 
the parameterization is somewhat different for 'correlated_k FU' than for all other 
cases because in the latter case the parameterization from the newer (March 2000) 
Fu and Liou code is used while otherwise the data are taken from the original 
Hu and Stamnes paper. Note that this parameterization has been
developed to calculate irradiances, hence it is less suitable for radiances.
This is due to the use of the Henyey-Greenstein phase function as an approximation
of the real Mie phase function.
@item mie
Use pre-calculated Mie tables; useful for @code{correlated_k}; 
the tables are expected in @code{data_files_path}/correlated_k/../ @*
For spectral or pseudo-spectral (@code{correlated_k sbdart}) calculations, 
a set of pre-calculated tables is also available; the wavelength grid points of these 
data has been carefully selected such that the extinction cross section, 
single scattering albedo, and the asymmetry parameter are accurate to 1% 
(compared to the fully-resolved Mie calculation) for all wavelengths
between 250nm and 100 micron. For spectral or pseudo-spectral
calculations @code{wc_properties_interpolate} has to be defined explicitely to
initiate the interpolation of the optical properties to the internal wavelength grid.
Please note that this option may be extremely memory-comsuming because for each 
internal wavelength a full set of Legendre moments of the phase function is 
stored (up to several thousands). The Mie tables are not part of the standard distribution 
(because of their large size) but they are freely available from http://www.libradtran.org. 
This is the correct option to calculate radiances, to be preferred over the 
Henyey-Greenstein approach of Hu and Stamnes (1993).
@item filename
Read optical properties from specified filename; file format is as produced 
by Frank Evans' @code{cloudprp}; for each of the internal (computational) wavelengths, 
a separate file is expected. Use only if you really know what you are doing
(this option is subject to change).
@end table
@item wc_properties_interpolate
Interpolate water cloud optical properties over wavelength; useful for precalculated 
optical property files defined with @code{wc_properties}. Please note that this option 
may be extremely memory-comsuming because for each internal wavelength a full set of 
Legendre moments of the phase function is stored (up to several thousands). 
@item wc_scale_gg
Scale the water cloud asymmetry factor for all wavelengths and altitudes
with a float between 0.0 and 1.0.
@item wc_scale_ssa
Scale the water cloud single scattering albedo for all wavelengths and altitudes
with a float between 0.0 and 1.0.
@item wc_set_gg
Set the water cloud asymmetry factor for all wavelengths and altitudes
to a float between -1.0 and 1.0. This option is useful only for monochromatic 
calculations or in wavelength regions where the optical properties of water clouds 
can be considered constant, e.g. the ultraviolet range.
@item wc_set_ssa
Set the water cloud single scattering albedo for all wavelengths
and altitudes to a float between 0.0 and 1.0. This option is useful only for monochromatic 
calculations or in wavelength regions where the optical properties of water clouds 
can be considered constant, e.g. the ultraviolet range.
@item wc_set_tau
Set the total water cloud optical thickness to a constant value for all wavelengths.
The optical thickness defined here is the integral from the surface at the 
user-defined @code{altitude} to TOA (top of atmosphere). This option is useful only 
for monochromatic calculations or in wavelength regions where the optical properties 
of water clouds can be considered constant, e.g. the ultraviolet range.
@item wc_set_tau550
Set the water cloud optical thickness at 550nm. The optical thickness defined here 
is the integral from the surface at the user-defined @code{altitude} 
to TOA (top of atmosphere). Other wavelengths are scaled accordingly.
Note that this option requires for technical reasons that the
wavelength interval defined by @code{wvn} does contain 550nm.
@item wvn
Set the wavelength range by specifying first and last wavelength in nm. 
The default output wavelength grid is that defined in @code{solar_file}, 
unless @code{spline} is specified. Note that the radiative transfer calculations 
are done on an internal grid which can be influenced with @code{transmittance_wl_file}
or @code{molecular_tau_file}
@item wvn_index
Set the wavelengths to be selected. To be used together with predefined wavelength grids,
such as @code{transmittance_wl_file} @code{molecular_tau_file} and particularly 
useful in combination with the @code{correlated_k} option where often only a 
specified number of wavelength bands is required. E.g., in combination with
@code{correlated_k AVHRR_KRATZ}, @code{wvn_index 15 15} will select wavelength
index 15 which corresponds to channel 4, or @code{wvn_index 10 14} will select 
those bands required for channel 3. Indices start from 1.
@item zout 
Output altitudes in km. One or more altitudes may be specified in increasing
magnitude. Output altitudes must be within the range defined in the
@code{atmosphere_file}. Note that @code{zout} does not restructure the
atmosphere model. Hence, if you specify @code{zout 0.730} and have your
atmosphere model in @code{atmosphere_file} go all the way down to 
sea level, i.e. 0.0km., output is presented at 0.730km and calculations
performed with an atmosphere between 0.0 and 0.730 km (and above of course). 
If you want calculations done for e.g. an elevated site you have to restructure
the atmosphere model and make sure it stops at the appropriate altitude.
This you may either due by editing the atmosphere file or by using 
@code{altitude}. Note that for @code{rte_solver polradtran} the atmosphere file must 
contain the altitudes specified by @code{zout}.
@end vtable
  @c11_03@ */

  /*  @11_02c@                
@cindex output, uvspec
@cindex uvspec, output
The uvspec output differs for the different solvers and is currently fixed 
for each solver. We hope to make it user controllable in the future.

@node DISORT, SDISORT and SPSDISORT
@subsubsection DISORT, SDISORT and SPSDISORT
For the disort, sdisort and spsdisort solvers @code{uvspec} outputs one block of data to 
standard output (stdout) for each wavelength. The format of the block is

@example
lambda rfldir rfldn flup uavgso uavgdn uavgup
@end example

@noindent if @code{umu} is not specified. If @code{umu} is specified the format
of the block is

@example
lambda rfldir rfldn flup uavgso uavgdn uavgup 
umu(0) u0u(umu(0)) 
umu(1) u0u(umu(1)) 
  .     .          
  .     .          
@end example

@noindent If both @code{umu} and @code{phi} is specified
the output format of each block is

@example
lambda rfldir rfldn flup uavgso uavgdn uavgup
                            phi(0)     ...        phi(m)
umu(0) u0u(umu(0))   uu(umu(0),phi(0)) ... uu(umu(0),phi(m))
umu(1) u0u(umu(1))   uu(umu(1),phi(0)) ... uu(umu(1),phi(m))
  .     .             .                     .
  .     .             .                     .
umu(n) u0u(umu(n))   uu(umu(n),phi(0)) ... uu(umu(n),phi(m))
@end example

@noindent and so on for each wavelength.

@node TWOSTR
@subsubsection TWOSTR
The format of the output line for the twostr solver is

@example
lambda rfldir rfldn flup uavg
@end example

for each wavelength.

@node POLRADTRAN
@subsubsection POLRADTRAN
The output from the @code{polradtran} solver depends on the number of
Stokes parameters, @code{polradtran_nstokes}. For each wavelength the 
output block is

@example
lambda down_flux(1) up_flux(1) ...  down_flux(is) up_flux(is)
@end example

@noindent if @code{phi} is not specified. Here @code{is} is the number
of Stokes parameters specified by @code{polradtran_nstokes}. If 
@code{phi} and @code{umu} are specified the block is

@example
lambda down_flux(1) up_flux(1) ...  down_flux(is) up_flux(is)
                             phi(0)                 ...                 phi(m)
Stokes vector I
umu(0) u0u(umu(0))   uu(umu(0),phi(0)) ... uu(umu(0),phi(m))
umu(1) u0u(umu(1))   uu(umu(1),phi(0)) ... uu(umu(1),phi(m))
  .     .             .                     .
  .     .             .                     .
umu(n) u0u(umu(n))   uu(umu(n),phi(0)) ... uu(umu(n),phi(m))
Stokes vector Q
  .       .                                          .	                    
  .       .                                          .	                    
@end example

@noindent Note that @code{polradtran} outputs the total (=direct+diffuse) 
downward flux. Also note that u0u is always zero for @code{polradtran}.

@node Description of symbols
@subsubsection Description of symbols

In the above output blocks the symbols used have the following meaning. 

@vtable @code
@item cmu 
Computational polar angles from @code{polradtran}.
@item down_flux, up_flux
The total (direct+diffuse) downward (down_flux) and upward (up_flux) irradiances.
Same units as extraterrestrial irradiance.
@item lambda
Wavelength (nm)
@item rfldir
Direct beam irradiance (same units as extraterrestrial irradiance, e.g mW/(m2 nm) if 
using the @file{atlas3} spectrum in the @file{data/solar_flux} directory.)
@item rfldn
Diffuse down irradiance, i.e. total minus direct beam (same units
as @code{rfldir}).
@item flup
Diffuse up irradiance (same units as @code{rfldir}).
@item uavg
The mean intensity. Proportional to the actinic flux. (same units as @code{rfldir}).
@item uavgso
Direct beam contribution to the mean intensity. (same units as @code{rfldir}).
@item uavgdn
Diffuse downward radiation contribution to the mean intensity. (same units as @code{rfldir}).
@item uavgup
Diffuse upward radiation contribution to the mean intensity. (same units as @code{rfldir}).
@item u0u
The azimuthally averaged intensity at @code{numu} user specified 
angles @code{umu}. (units of e.g. mW/(m2 nm sr) if using the @file{atlas3} spectrum 
in the @file{data/solar_flux} directory.)  
@item uu
The radiance (intensity) at @code{umu} and @code{phi} user specified 
angles. (units of e.g. mW/(m2 nm sr) if using the @file{atlas3} spectrum 
in the @file{data/solar_flux} directory.)  
@item uu_down, uu_up
The downwelling and upwelling radiances (intensity) at @code{cmu} and 
@code{phi} angles. (units of e.g. mW/(m2 nm sr) if using the @file{atlas3} spectrum 
in the @file{data/solar_flux} directory.)  

@end vtable

@noindent The total downward irradiance is given by

@example
irr_down = rfldir + rfldn
@end example

@noindent The total mean intensity is given by

@example
uavg = uavgso + uavgdn + uavgup
@end example

@noindent If @code{deltam} is on it does not make sense to look at the individual 
contributions to @code{uavg} since they are delta-M scaled.

  @c11_02@ */

  /*  @11_01c@                

uvspec is controlled in a user-friendly way. The control options are
named in a (hopefully) intuitive way. In the following, several
examples are given, how to create an input file, how to define a
cloudless sky atmosphere, how to add aerosols and clouds, etc. All
following examples are taken from the libRadtran examples directory
and are part of the uvspec self-check. For a complete listing and
explanation of all input options, have a look at section 
@ref{Complete description of input parameters}.

The uvspec input file consists of single line entries, each making
up a complete input to the uvspec program. First on the line comes
the parameter name, followed by one or more parameter values. The 
parameter name and the parameter values are separated by white space.
Filenames are entered without any surrounding single or double quotes.
Comments are introduced by a @code{#}. Blank lines are ignored. 

@subsubsection Cloudless, aerosol-free atmosphere

The simplest possible input file contains only a few lines:
@example
@cartouche
@include ../examples/UVSPEC_SIMPLE.INP
@end cartouche
@end example

The first three statements define the location of some data files: the
uvspec data path (@code{data_files_path}) where all internal files are
expected, the atmospheric profile
(@code{atmosphere_file}), and the extraterrestrial spectrum
(@code{solar_file}). The last line defines the desired wavelength
range which is a monochromatic data point in this example. All other data
which are not explicitely mentioned assume a default value which is
"0" in most cases. Here, the solar zenith angle is 0, the
surface albedo is 0, and the atmosphere does not contain clouds 
nor aerosols. Pressure, temperature, ozone concentration, etc. are 
read from @code{atmosphere_file}.

An example of a more complete input file for a clear sky atmosphere is

@example
@cartouche
@include ../examples/UVSPEC_CLEAR.INP
@end cartouche
@end example

@noindent 
The atmosphere model, i.e. pressure, temperature, and ozone concentration
profiles are read from @code{../data/atmmod/afglus.dat}. The 
extraterrestrial solar flux is read from the file 
@code{../data/solar_flux/atlas_plus_modtran}.

A wavelength dependent @b{surface albedo} may be specified 
using @code{albedo_file} instead of @code{albedo}. Non-Lambertian
surface reflectance (BRDF) for vegetation and water may also be
defined (please note that these require the use of 
@code{rte_solver disort2}. 
The BRDF of Vegetation is specified using @code{rpv_rho0},
@code{rpv_k}, and @code{rpv_theta}, following the definition of 
Rahman et al. (1993). Wavelength-dependent BRDF for vegetation can
be defined with @code{rpv_file}. The BRDF of water surfaces is parameterized
following Cox and Munk (1954a, 1954b) and Nakajima (1983). The
respective parameters are the wind speed @code{cox_and_munk_u10}, 
the pigment concentration @code{cox_and_munk_pcl}, and the salinity
@code{cox_and_munk_sal}. A complete description of these parameters is
given in section @ref{Complete description of input parameters}.

It is helpful to know some details about the @b{input/output
wavelength resolution} in uvspec and how it can be influenced by the
user. Basically there are three independent wavelength grids, the
@strong{input grid}, the @strong{internal grid}, and the 
@strong{output grid}. The essential thing to know is that the
internal grid is chosen by uvspec itself in a reasonable way, if not
explicitely defined in the input file with @code{transmittance_wl_file}
or @code{molecular_tau_file}.
The output grid is completely independent of the internal grid and 
is entirely defined by the @code{solar_file}. The wavelength grid of
all other input data (e.g. albedo, optical properties of aerosols and 
clouds, etc) is also completely independent. These data are 
automatically interpolated to the resolution of the internal
wavelength grid. Hence, only two constraints are set to the gridding
of the input data: (1), the wavelength range has to cover all internal
grid points; and (2), it should be chosen in a reasonable manner to
allow reasonable interpolation (which essentially means, dense enough).

In the ultraviolet/visible, uvspec uses an internal grid with a step
with of 0.5nm below 350nm and 1nm above 350nm. This is a conservative
choice which fully resolves the broad ozone absorption bands and the
slowly varying Rayleigh, aerosol, and cloud extinctions. The idea is
outlined in the following figure which is taken from Mayer et
al. (1997):

@image{./transpz,}

The transmittance (or reflectance) is calculated on a moderate
resolution grid which reduces the number of calls to the
@code{rte_solver} and hence the computational time. Then, the
transmittance is interpolated to the wavelengths in the @code{solar_file}
(which is usually defined with higher spectral resolution), multiplied 
with the extraterrestrial irradiance, and possibly post-processed.
Hence, the wavelength in the output spectrum are those
contained in the @code{solar_file} which has two important
implications: (1) Only those wavelengths are output that are contained
in the @code{solar_file}. If e.g. a monochromatic calculation is 
defined by setting '@code{wvn 327.14 327.14}', there will only be output
if the wavelength 327.14 is explicitely listed in @code{solar_file};
(2) this is also true at thermal wavelengths where the
extraterrestrial irradiance is zero; hence, even for a calculation in the
thermal range a @code{solar_file} can be specified which defines the
output grid in the first column and arbitrary values in the second
column. Keeping these points in mind, @code{solar_file} is a
convenient way to define an arbitrary output grid. 
@code{solar_file} may be omitted for thermal radiation calculations 
(@code{source thermal}) as well as for @code{transmittance} 
and @code{reflectivity} calculations. If omitted, the output grid
equals the internal wavelength grid.

If required, a @b{user-defined internal grid} can be specified with 
@code{transmittance_wl_file} or @code{molecular_tau_file}. 
Note that this is a way to speed up the calculation considerably. 
E.g., for some applications the internal grid in the UV-A and visible 
can be set to 10nm which would reduce computational times 
by up to a factor of 10. 

Things are completely different if one of the @code{correlated_k}
parameterizations is selected (see below). In this case all flexibility
is taken away from the user which is an inherent feature of the
k distribution method. Internal grid as well as the extraterrestrial
file are in this case defined by the choice of the parameterization itself.


@subsubsection Spectral resolution

@code{uvspec} offers four different ways of spectral calculations:
@enumerate
@item
@strong{Spectrally resolved calculation} in the UV and visible spectral ranges;
@item 
@strong{Line-by-line calculation} with user-defined molecular absorption data;
@item 
@strong{The correlated-k method}.
@item 
@strong{Pseudo-spectral calculation} with exponential-sum-fit,
adopted from SBDART (Ricchiazzi et al., 1998);
@end enumerate

The choice of the method is determined by the problem and the decision
is therefore entirely up to the user. The spectrally resolved
calculation and the line-by-line calculation are more or less exact
methods while the correlated-k distribution and the pseudo-spectral
calculation are approximations that provide a compromise between speed
and accuracy. In the following it is briefly described which method
fits which purpose: 

A @strong{spectrally resolved calculation} is the most straightforward
way, and will be the choice for all users interested in the
ultraviolet and visible spectral ranges. In the UV/vis gas absorption
generally occurs in broad bands with only slow spectral variation, the
most important of these being the Hartley, Huggins, and Chappuis bands
of ozone. Hence, a radiative transfer calculation every 1nm usually is
sufficient to fully  resolve any spectral variation using the method
described in the last section. Absorption cross sections for various
species are included, among them the most important O3 and NO2.

In the infrared, however, molecular absorption spectra are
characterized by thousands of narrow absorption lines. There are  
two ways to treat these, either by highly resolved spectral
calculations, so-called @strong{line-by-line} calculations, or by a band
parameterization. Concerning line-by-line, uvspec offers the
possibility to define a spectrally resolved absorption cross section
profile using @code{molecular_tau_file}. There is no option in
libRadtran to generate such a @code{molecular_tau_file}, because 
(1) the HITRAN database which forms the basis for such calculations
amounts to about 100 MByte which are updated continuously; and (2), 
there are sophisticated line-by-line programs available, like
e.g. @code{genln2} (Edwards, 1992). Using @code{genln2} it is
straightforward to create the input for uvspec line-by-line
calculations. It is also planned to make line-by-line cross sections
available for the six standard profiles that come with
libRadtran. The following figure shows an example of a line-by-line
calculation of the atmospheric transmittance in two selected solar
and thermal spectral ranges, the O2A-absorption band around 760nm and
a region within the infrared window around 10 micron.

@image{./lblo2a}

All spectral lines in the left figure are due to absorption by oxygen,
while the ones in the left figure are due to ozone, water vapour, and
CO2. Line-by-line is obviously the exact way for radiation
calculations. For most applications, however, line-by-line is far too
slow. Here one 
needs a band parameterization, and the most accurate of these is the
so-called @strong{correlated-k approximation}. uvspec contains several
correlated-k parameterizations which are invoked with 
@code{correlated_k}, in particular Kato et al. (1999), 
Fu and Liou (1992/93), Kratz (1995), as well as the possibility to
specify a user-defined one. Kato et al. (1999) is a accurate parameterization 
for the solar spectral range. @code{uvspec} contains three different versions:
@vtable @code
@item Kato 
The original tables provide by Seiji Kato which should correspond to the 
full version described in Kato et al. (1999); 575 subbands total, that is,
575 calls to the @code{rte_solver}
@item Kato2
A new, optimized version of the tables, provided by Seiji Kato, 2003, with only
148 subbands (that is, calls to the @code{rte_solver}); the uncertainty is only 
slightly higher than @code{Kato}; the absorption coefficients are based on HITRAN 2000.
@item Kato2.96
Similar to @code{Kato2} but based on HITRAN96.
@end vtable
The following Figure shows a comparison between the three parameterization which 
are part of libRadtran and the data from Figure 3 by Kato et al. [1999]. It is 
immediately obvious that the uncertainty is high for all bands above 2.5 micron 
which is probably due to the treatment of band overlap. For this reasons, the 
results for the indiviudal bands should not be trusted while the integrated 
shortwave radiation (the sum of all 32 bands) is calculated with high accuracy because
(1) the bands above 2.5 micron contribute only little to the integrated irradiance; 
and (2) errors are random and cancel each other to some degree.

@image{./figure3}

For more information on these parameterizations please refer to the mentioned
publications. Correlated-k is a powerful way to calculate spectrally
integrated quantities, however, it takes away some flexibility. In
particular, this means that the wavelength grid is no longer chosen by
the user but by the parameterization, that is, by @code{uvspec}. 
The @code{uvspec} output is then no longer spectral quantities,
e.g. W / (m2 nm), but integrated over the spectral bands, e.g. 
W / m2.

A simple but complete example for a correlated-k approximation 
of the solar spectrum:

@example
@cartouche
@include ../examples/UVSPEC_KATO.INP
@end cartouche
@end example

Here, the solar spectrum is split up into 32 bands according to 
Kato et al. (1999). In order to calculate integrated shortwave 
irradiance, simply sum the outputs, or even simpler, add 
@code{output sum} to the input file.

For @strong{pseudo-spectral calculations} in the whole spectral range, we have
adopted the molecular absorption parameterization from SBDART by 
Ricchiazzi et al. (1998). The respective section of this paper says:

@quotation
SBDART relies on low-resolution band models developed for the LOWTRAN
7 atmospheric trans-mission code (Pierluissi and Peng, 1985).  
These models provide clear-sky atmospheric transmission from 0 to
50000 cm-1 and include the effects of all radiatively active molecular
species found in the earth s atmosphere. The models are derived from
detailed line-by-line calculations that are degraded to 20 cm-1
resolution for use in LOWTRAN. This translates to a wavelength
resolution of about 5 nm in the visible and about 200 nm in the
thermal infrared. These band models represent rather large wavelength
bands, and the transmission functions do not necessarily follow Beers
Law. This means that the fractional transmission through a slab of
material depends not only on the slab thickness, but also on the
amount of material penetrated before entering the slab. Since the
radiative transfer equation solved by SBDART assumes Beers Law
behavior, it is necessary to express the transmission as the sum of
several exponential functions (Wiscombe and Evans, 1977). SBDART uses
a three-term exponential fit, which was also obtained from LOWTRAN
7. Each term in the exponential fit implies a separate solution of the
radiation transfer equation. Hence, the RT equation solver only needs
to be invoked three times for each spectral increment. This is a great
computational economy compared to a higher order fitting polynomial,
but it may also be a source of significant error. 
@end quotation

The SBDART gas parameterization is invoked with
@code{correlated_k SBDART}. The spectral resolution may be arbitrarily
chosen by the user. If not explicitely defined with
@code{transmittance_wl_file}, an internal grid with a step width of
0.5nm below 350nm and 1nm above 350nm is chosen which is practically
overkill for most applications in the infrared. An extraterrestrial
spectrum covering the complete solar range is provided at two
different resolutions, @code{data/solar_flux/kurudz_1.0nm.dat} and 
@code{data/solar_flux/kurudz_0.1nm.dat}. An example for the solar
range is shown in @code{examples/UVSPEC_SBDART_SOLAR.INP}:

@example
@cartouche
@include ../examples/UVSPEC_SBDART_SOLAR.INP
@end cartouche
@end example

while @code{examples/UVSPEC_SBDART_THERMAL.INP} shows how to do a
thermal calculation:

@example
@cartouche
@include ../examples/UVSPEC_SBDART_THERMAL.INP
@end cartouche
@end example

@image{./sbdart}

The figure shows the results of the solar and thermal
calculations. The water vapour absorption bands in the solar range
are clearly visible, as is the absorption window around 10 micron in
the thermal. Please note the following points:
@itemize @bullet
@item 
Thermal radiation is per default output in W/(m2 cm-1), see 
documentation of @code{thermal_bandwith} and
@code{thermal_bands_file}. To convert e.g. to W/(m2 nm) multiply with  
k/lambda where k is the wavenumber [cm-1] and lambda is the 
wavelength [nm]. To calculate band-integrated thermal quantities
please consider @code{thermal_bands_file}.
@item 
Even though no extraterrestrial irradiance is required, a
@code{solar_file} may be specified for the thermal case. The reason
is that, as explained initially, the @code{solar_file} defines the
output grid. The second column in @code{solar_file} can be chosen
arbitrarily in this case because it is ignored. 
@item 
For the choice of the wavelength grid for the calculation
(@code{transmittance_wl_grid}) please consider that the resolution of
the absorption parameterization is 5cm-1 which converts to 0.3nm at
750nm and to 50nm at 10 micron. Choosing higher resolutions for 
the internal wavelength grid (@code{transmittance_wl_file}) is usually
a waste of computational time.
@item
Please also make sure to choose a fine enough spectral resolution in
order to capture all absorption features.
@end itemize

The following figure shows two selected wavelength intervals of the
solar and thermal range, to demonstrate the spectral resolution of the
SBDART absorption parameterization.

@image{./sbdarto2a}

The resolution is about 5cm-1 which translates to about 0.3nm in the
left figure (oxygen A-band) and 50nm in the right figure (ozone
absorption band in the atmospheric window). Compare this figure to the
above line-by-line example to get an impression about the differences
between both methods.

@subsubsection Aerosol

All options to set up and modify aerosol properties start with 
@code{aerosol_}.
Aerosols may be specified in a hierarchical way. The most simple way to define 
an aerosol is by the command @code{aerosol_default} which will set up the
aerosol model by
E.P. Shettle, "Models of aerosols, clouds and precipitation for 
atmospheric propagation studies", in "Atmospheric propagation in the 
uv, visible, ir and mm-region and related system aspects", 
AGARD Conference Proceedings (454), 1989. The default properties are 
a rural type aerosol in the boundary layer, background aerosol above 2km,
spring-summer conditions and a visibility of 50km. These settings may
be modified with @code{aerosol_haze}, @code{aerosol_vulcan}, @code{aerosol_season},
and @code{aerosol_visibility}. More information can be introduced step by step,
overwriting the default parameters. @code{aerosol_tau_file}, @code{aerosol_ssa_file},
and @code{aerosol_gg_file}, can be used to define the profiles of optical thickness,
single scattering albedo, and asymmetry parameter. The integrated optical thickness 
can be set to a constant value using @code{aerosol_set_tau} or scaled 
with @code{aerosol_scale_tau}. The single scattering albedo may be scaled by 
@code{aerosol_scale_ssa} or set to a constant value by @code{aerosol_set_ssa}.  
The aerosol asymmetry factor may be set by @code{aerosol_set_gg}. 
The wavelength dependence of the aerosol optical depth is specified using the 
@code{aerosol_angstrom} parameter. @code{aerosol_moments_file} allows specification of 
the scattering phase function. If microphysical properties are available these 
may be introduced by defining the complex index of refraction @code{aerosol_refrac_index} 
or @code{aerosol_refrac_file} and the size distribution @code{aerosol_sizedist_file}. 
Finally, one may define the aerosol optical properties of each layer explicitely 
using @code{aerosol_files}.

The following list is an overview of the aerosol description parameters. The 
entries are arranged in a way that a parameter 'overwrites' all values higher up 
in the list.

@table @code
@item aerosol_default
Generate default aerosol according to Shettle (1989)
@item aerosol_vulcan, aerosol_haze, aerosol_season, aerosol_visibility
Set Shettle (1989) aerosol properties (aerosol type, visibility)
@item aerosol_files
Specify optical properties of each layer explicitly, that is, extinction coefficient,
single scattering albedo, and the moments of the phase function
(everything as a function of wavelength).
@item aerosol_tau_file, aerosol_ssa_file, aerosol_gg_file
Overwrite profiles of optical thickness, single scattering albedo, and
asymmetry parameter
@item aerosol_moments_file
Specify a phase function to be used instead of the Henyey-Greenstein phase function
@item aerosol_refrac_index, aerosol_refrac file, aerosol_sizedist_file
Calculate optical properties from size distribution and index of refraction using 
Mie theory. Here is an exception from the rule that ALL values defined above are 
overwritten because the optical thickness profile is re-scaled so that the 
optical thickness at the first internal wavelength is unchanged. It is done 
that way to give the user an easy means of scaling the specifying the optical 
thickness at a given wavelength. 
@item aerosol_set_gg, aerosol_set_ssa, aerosol_scale_ssa, aerosol_set_tau, aerosol_scale_tau
Overwrite profiles of asymmetry parameter and single scattering albedo
@item aerosol_angstrom
Overwrite the profile of the optical thickness.
@end table

An example for a uvspec aerosol description is

@example
@cartouche
@include ../examples/UVSPEC_AEROSOL.INP
@end cartouche
@end example

@noindent 
By combining this with the clear sky example given above a complete uvspec
input file including aerosol is constructed.


@subsubsection Water clouds

All options to set up and modify water cloud properties 
start with @code{wc_}. 

The easiest way to define a water cloud is to specify a 
@code{wc_file} which defines the liquid water content 
and effective droplet radius at each model level. By combining the
following lines with the clear sky example given above a complete
uvspec input file including water clouds is constructed. 

@example
@cartouche
@include ../examples/UVSPEC_WC.INP
@end cartouche
@end example

@noindent 
The file @code{WC.DAT} is found in the examples directory.
Note that, other than for the aerosol, cloud properties are defined 
at each model level, rather than for a model layer. To define cloud
properties per layer, one needs to specify @code{wc_layer}.

There are different ways to convert the microphysical properties to
optical properties. Either a parameterization is used, like the one 
by Hu and Stamnes (1993) (which is the default), or by Mie
calculations. The latter are very time-consuming, hence we decided not
to include these online into uvspec but rather have an option to read
in pre-calculated Mie tables. The option @code{wc_properties} controls
the method: @code{hu} selects the Hu and Stamnes (1993)
parameterization, @code{mie} selects pre-calculated Mie tables which
are available at http://www.libradtran.org. Finally, there is the
option to define an arbitrary file which is in the format as generated
by Frank Evans' @code{cloudprp} which comes with SHDOM, see 
http://nit.colorado.edu/~evans/shdom.html.

As for the aerosol, there are several options to modify the optical
properties of the clouds. And of course there is also the option of 
defining all cloud properties explicitely using @code{wc_files}.


@subsubsection Ice clouds

Ice clouds are generated in a similar way to water clouds.
All options to set up and modify ice cloud properties 
start with @code{ic_}. The main difference between water and ice clouds
is that the latter usually consist of non-spherical particles. Hence,
the conversion from microphysical to optical properties is much less
defined, and several parameterizations are available.


@subsubsection Calculation of radiances

To calculate radiances the following lines will do the job when combined
with the clear sky example above

@example
@cartouche
@include ../examples/UVSPEC_RADIANCES.INP
@end cartouche
@end example

More examples with output are found in the @file{examples} directory.

      @c11_01@   */

  /***************** End of documentation *********************/

  int i=0, c=0;
  int brdftype=0;
  int status=0;

  char base[255]="";
  
  /* Initialization */

  /* allocate memory for filenames */

  Input.filename         = calloc (FN_NN,     sizeof(char *));
  Input.aer.filename     = calloc (FN_AER_NN, sizeof(char *));
  Input.atm.filename     = calloc (MOL_NN,    sizeof(char *));
  Input.rte.mc.filename  = calloc (FN_MC_NN,  sizeof(char *));
  Output.crs.filename    = calloc (MOL_NN,    sizeof(char *));

  for (i=0; i<FN_NN; i++)
    Input.filename[i]     = calloc(FILENAME_MAX+1, sizeof(char));
  
  for (i=0; i<FN_AER_NN; i++) 
    Input.aer.filename[i] = calloc(FILENAME_MAX+1, sizeof(char));

  for (i=0; i<MOL_NN; i++) 
    Input.atm.filename[i] = calloc(FILENAME_MAX+1, sizeof(char));

  for (i=0; i<FN_MC_NN; i++)
    Input.rte.mc.filename[i] = calloc(FILENAME_MAX+1, sizeof(char));

  for (i=0; i<MOL_NN; i++) 
    Output.crs.filename[i] = calloc(FILENAME_MAX+1, sizeof(char));


  Input.quiet             = 0;
  Input.verbose           = 0;
  Input.flexstor          = 0;
  Input.header            = 0;
  Input.source            = SRC_SOLAR;  /* default: solar source     */
  Input.bandwidth         = 1.0;        /* default: bandwidth in cm-1 */
  Output.print_phi        = 1;          /* default: print radiances   */
  Input.ck_scheme         = CK_NONE;
  Input.ck_h2ocont        = 1;
  Input.cloudcover        = -999;
  Input.wcipa             = 0;
  Input.icipa             = 0;
  Input.convolve          = 0;
  Input.spline            = 0;
  Input.ozone_spectrum    = O3_MOLINA;
  Input.no2_spectrum      = NO2_GOME;
  Input.bro_spectrum      = BRO_IASB;
  Input.oclo_spectrum     = OCLO_WAHNER;
  Input.hcho_spectrum     = HCHO_IASB;
  Input.day_of_year       = -999;
  Input.precip_water      = -999;
  Input.pressure          = -999;
  Input.temperature       = -999;

  Input.aer.standard      = 0;
  Input.aer.spec          = 0;
  Input.aer.alpha         = 0.0;
  Input.aer.beta          = -999.;
  Input.aer.haze          = 0;
  Input.aer.scale_ssa     = -999.;
  Input.aer.scale_tau     = -999.;
  Input.aer.tau550        = -999;
  Input.aer.set_gg        = -999.;
  Input.aer.set_tau       = -999.;
  Input.aer.set_ssa       = -1.;
  Input.aer.seasn         = 0;
  Input.aer.visibility    = -999.;
  Input.aer.vulcan        = 0;

  Input.atm.albedo        = 0.0;
  Input.atm.altitude      = -999;
  Input.atm.altitude_dz   = 0.0;
  Input.atm.sza           = 0.0;

  Input.brdf              = BRDF_NONE;
  Input.cm.u10            = -999.;
  Input.cm.pcl            = -999.;
  Input.cm.sal            = -999.;

  Input.calibration       = OUTCAL_ABSOLUTE;
  Input.rte.fisot         = 0;
  Input.rte.ichap         = 1;
  Input.rte.maxphi        = 0;
  Input.rte.nscat         = 20;
  Input.rte.nrefrac       = 0;
  Input.rte.nstr          = 6;
  Input.rte.solver        = DISORT2;
  Input.rte.pol_stokes[0] = 'I';
  Input.rte.pol_stokes[1] = 'Q';
  Input.rte.pol_stokes[2] = 'U';
  Input.rte.pol_stokes[3] = 'V';

  Input.rte.mc.absorption = MCABS_NONE;
  Input.rte.mc.photons    = 100000;
  Input.rte.mc.momcld     = MOMCLD_NONE;
  Input.rte.mc.Nx_sample  = 0;
  Input.rte.mc.Ny_sample  = 0;
  Input.rte.mc.Nx_cloud   = 0;
  Input.rte.mc.Ny_cloud   = 0;
  Input.rte.mc.Nz_cloud   = 0;
  Input.rte.mc.passback3D = 0;
  Input.rte.mc.ipa        = 0;
  Input.rte.mc.truncate   = -999;
  Input.rte.mc.bcond      = MCBCOND_PERIODIC;

  Input.wl.start          = -999;
  Input.wl.end            = -999;
  Input.wl.start_index    = -999;
  Input.wl.end_index      = -999;

  Input.rte.nphi          = 0;
  Input.rte.numu          = 0;
  Input.rte.nprndis       = 0;
  Input.rte.deltam        = 1;
  Input.rte.pol_aziorder  = 0;
  Input.rte.pol_max_delta_tau = 1e-5;
  Input.rte.pol_nstokes   = 1;
  Input.rte.pol_quad_type = "G";
  Input.rte.pol_src_code  = 1;
  Input.rte.reverse       = 0;
  Input.atm.nzout         = 0;
  Input.wc.tau            = -999;
  Input.wc.gg             = -999;
  Input.wc.ssa            = -999;
  Input.wc.scale_gg       = -999;
  Input.wc.scale_ssa      = -999;
  Input.wc.tau550         = -999;
  Input.wc.layer          = 0;
  Input.wc.properties     = WCPROP_NONE;
  Input.wc.interpolate    = 0;
  Input.ic.tau            = -999;
  Input.ic.gg             = -999;
  Input.ic.ssa            = -999;
  Input.ic.scale_gg       = -999;
  Input.ic.scale_ssa      = -999;
  Input.ic.tau550         = -999;
  Input.ic.layer          = 0;
  Input.ic.properties     = ICPROP_NONE;
  Input.ic.interpolate    = 0;
  Input.ic.habit          = IC_HABIT_SOLIDCOLUMN;
  Input.absorption        = 1;
  Input.molabs            = MOLABS_CALC;
  Input.rayleigh          = RAYLEIGH_CALC;
  Input.scattering        = 1;
  Input.mixing_ratio = calloc(MX_NN, sizeof(float));
  for (i=0; i<MX_NN; i++)
    Input.mixing_ratio[i] = -999;
  Input.atm.column       = calloc(MOL_NN, sizeof(float));
  Input.atm.scale_factor = calloc(MOL_NN, sizeof(float));
  Input.atm.unit         = calloc(MOL_NN, sizeof(int));
  for (i=0; i<MOL_NN; i++) {
    Input.atm.column[i] = -999;
    Input.atm.scale_factor[i] = 1.0;
  }
  Input.atm.unit[MOL_O3]   = DU;
  Input.atm.unit[MOL_O2]   = CM_2;
  Input.atm.unit[MOL_H2O]  = CM_2;
  Input.atm.unit[MOL_CO2]  = CM_2;
  Input.atm.unit[MOL_NO2]  = CM_2;
  Input.atm.unit[MOL_BRO]  = CM_2;
  Input.atm.unit[MOL_OCLO] = CM_2;
  Input.atm.unit[MOL_HCHO] = CM_2;

  Input.processing         = PROCESS_NONE;

  while ((c=getopt (argc, argv, "hv")) != EOF)  {
    switch(c)  {
    case 'h': 
      usage();
      return (-1);
      break;
    case 'v': 
      version();
      return (-1);
      break;
    default:
      usage();
      return (-1);
    }
  }

  /* Read the input file */

  yylex();

  if (ierror!=0)  {
    fprintf (stderr, "Error in input file! Please check your input file and retry!\n");
    return -1;
  }

  /* if uvspec is told to be verbose, it cannot be quiet at the same time */
  if (Input.verbose)
    Input.quiet = 0;


  /* To calculate radiances, we want to increase nstr to at least 16 */
  if (Input.rte.numu > 0)
    if (Input.rte.nstr<16)  {
      if (!Input.quiet)
        fprintf (stderr, " ... setting nstr to 16 for radiance calculation!\n");
	Input.rte.nstr=16;
    }


  if (Input.rte.numu>0 && Input.rte.solver==DISORT) {
    fprintf (stderr, "\n");
    fprintf (stderr, "***  Warning: You want to calculate radiances with DISORT.\n");
    fprintf (stderr, "***  It is strongly recommended to use DISORT2 in this case because\n");
    fprintf (stderr, "***  the older DISORT version uses only the first NSTR moments of the\n");
    fprintf (stderr, "***  phase function which might result in really bad distortions.\n");
    fprintf (stderr, "***  DISORT2 is set up to consider at least the first 1024 moments.\n");
    fprintf (stderr, "\n");
  }



  if (Input.atm.nzout == 0) {
    Input.atm.nzout    = 1;
    Input.atm.zout_sur = (float *) calloc (Input.atm.nzout, sizeof(float));

    if (Input.rte.solver == MONTECARLO)
      Input.atm.zout_sur[0] = -999.0;
    else 
      Input.atm.zout_sur[0] = 0.0;
  }


  /* surface temperature */
  Output.surface_temperature = Input.temperature;


  /* no user-defined transmittance grid if correlated-k */
  if ((Input.ck_scheme!=CK_NONE && Input.ck_scheme!=CK_SBDART) && strlen(Input.filename[FN_WLTRANS])>0) {
    strcpy (Input.filename[FN_WLTRANS], "");
    fprintf (stderr, " ... option transmittance_wl_file ignored\n");
    fprintf (stderr, " ... (does not make sense with correlated-k)\n");
  }


  /* Default MYSTIC output filename */
  if (strlen(Input.rte.mc.filename[FN_MC_BASENAME]) == 0)
    strcpy (Input.rte.mc.filename[FN_MC_BASENAME], "mc");
  

  /* MYSTIC 3D fields passed back to uvspec ? */
  if (Input.atm.nzout>0 && Input.rte.mc.Nx_sample>0 && Input.rte.mc.Ny_sample>0 
      && Input.rte.solver==MONTECARLO)
    Input.rte.mc.passback3D=1;

  if (strlen(Input.rte.mc.filename[FN_MC_MOMCLD])>0 && strlen(Input.rte.mc.filename[FN_MC_PCLD])>0) {
    fprintf (stderr, "Error. Please specify either mc_momcld_file or mc_pcld_file but\n");
    fprintf (stderr, "not both at the same time!\n");
    return -1;
  }


  if (strlen(Input.filename[FN_WCLOUD]) > 0 && Input.wc.properties == WCPROP_NONE)  {
    Input.wc.properties = WCPROP_HU;

    if (!Input.quiet)
      fprintf (stderr, " ... using default water cloud properties, Hu and Stamnes (1993)\n");
  }
  
  if (strlen(Input.filename[FN_ICLOUD]) > 0 && Input.ic.properties == ICPROP_NONE)  {
    Input.ic.properties = ICPROP_FU;

    if (!Input.quiet)
      fprintf (stderr, " ... using default ice cloud properties, Fu et al. (1996/98)\n");
  }

  switch (Input.wc.properties) {
  case WCPROP_MIE:
  case WCPROP_FILE:
  case WCPROP_EXPLICIT:
    if (Input.rte.solver==DISORT) {
      fprintf (stderr, "\n");
      fprintf (stderr, "***  Warning: You specified a tabulated phase function for water clouds.\n");
      fprintf (stderr, "***  It is strongly recommended to use DISORT2 in this case because\n");
      fprintf (stderr, "***  the older DISORT version uses only the first NSTR moments of the\n");
      fprintf (stderr, "***  phase function which might result in really bad distortions.\n");
      fprintf (stderr, "***  DISORT2 consideres all moments correctly.\n");
      fprintf (stderr, "\n\n");
    }
    break;
  default:
    break;
  }


  switch (Input.ic.properties) {
  case ICPROP_MIE:
  case ICPROP_FILE:
  case ICPROP_EXPLICIT:
    if (Input.rte.solver==DISORT) {
      fprintf (stderr, "\n");
      fprintf (stderr, "***  Warning: You specified a tabulated phase function for ice clouds.\n");
      fprintf (stderr, "***  It is strongly recommended to use DISORT2 in this case because\n");
      fprintf (stderr, "***  the older DISORT version uses only the first NSTR moments of the\n");
      fprintf (stderr, "***  phase function which might result in really bad distortions.\n");
      fprintf (stderr, "***  DISORT2 consideres all moments correctly");
      fprintf (stderr, "\n\n");
      }
      break;
    case ICPROP_YANG:
#if HAVE_YANG56
#else
     fprintf (stderr, " ***********************************************************************\n");
     fprintf (stderr, " * You have built uvspec without Yang et al. [2000] support and hence  *\n");
     fprintf (stderr, " * cannot use 'ic_properties yang'.                                    *\n");
     fprintf (stderr, " ***********************************************************************\n");
     return -1;
#endif
    default:
    break;
  }

  /* water cloud properties */
  if (Input.wc.properties == WCPROP_MIE) {
    switch (Input.ck_scheme) {
    case CK_KATO:
    case CK_KATO2:
    case CK_KATO2_96:
      if (Input.wc.interpolate)  {
        fprintf (stderr, "* It does not make sense to use \"wc_properties mie\" together with\n");
        fprintf (stderr, "* wc_properties_interpolate for \"correlated_k KATO\" because\n");
	fprintf (stderr, "* in the latter case the cloud properties are pre-calculated\n");
	fprintf (stderr, "* for the required wavelength and interpolation is not useful.\n");
	fprintf (stderr, "* Please remove 'wc_properties_interpolate' from the uvspec input.\n");
	return -1;
      }

      strcpy (Input.filename[FN_WCLOUD_PROP], Input.filename[FN_PATH]);
      strcat (Input.filename[FN_WCLOUD_PROP], "/correlated_k/kato/");
      strcat (Input.filename[FN_WCLOUD_PROP], "wc.");
      break;	    
		    
    case CK_FU:	    
      if (Input.wc.interpolate)  {
        fprintf (stderr, "* It does not make sense to use \"wc_properties mie\" together with\n");
        fprintf (stderr, "* wc_properties_interpolate for \"correlated_k FU\" because\n");
	fprintf (stderr, "* in the latter case the cloud properties are pre-calculated\n");
	fprintf (stderr, "* for the required wavelength and interpolation is not useful.\n");
	fprintf (stderr, "* Please remove 'wc_properties_interpolate' from the uvspec input.\n");
	return -1;
      }

      strcpy (Input.filename[FN_WCLOUD_PROP], Input.filename[FN_PATH]);
      strcat (Input.filename[FN_WCLOUD_PROP], "/correlated_k/fu/");
      strcat (Input.filename[FN_WCLOUD_PROP], "wc.");
      break;
      
    case CK_AVHRR_KRATZ:
      if (Input.wc.interpolate)  {
        fprintf (stderr, "* It does not make sense to use \"wc_properties mie\" together with\n");
        fprintf (stderr, "* wc_properties_interpolate for \"correlated_k AVHRR_KRATZ\" because\n");
	fprintf (stderr, "* in the latter case the cloud properties are pre-calculated\n");
	fprintf (stderr, "* for the required wavelength and interpolation is not useful.\n");
	fprintf (stderr, "* Please remove 'wc_properties_interpolate' from the uvspec input.\n");
	return -1;
      }

      strcpy (Input.filename[FN_WCLOUD_PROP], Input.filename[FN_PATH]);
      strcat (Input.filename[FN_WCLOUD_PROP], "/correlated_k/kratz/");
      strcat (Input.filename[FN_WCLOUD_PROP], "wc.");
      break;

    case CK_NONE:
    case CK_SBDART:
      if (!Input.wc.interpolate)  {
        fprintf (stderr, "* 'wc_properties mie' for a spectral calculation requires that the\n");
        fprintf (stderr, "* cloud optical properties are interpolated to the internal wavelength\n");
	fprintf (stderr, "* grid. Depending on the number of grid points this may require huge\n");
	fprintf (stderr, "* amounts of memory and some additional computational time. To use this\n");
	fprintf (stderr, "* option, please specify explicitely 'wc_properties_interpolate' \n");
	fprintf (stderr, "* in the uvspec input!\n");
	return -1;
      }
      else {
        strcpy (Input.filename[FN_WCLOUD_PROP], Input.filename[FN_PATH]);
        strcat (Input.filename[FN_WCLOUD_PROP], "/wc/mie/");
        strcat (Input.filename[FN_WCLOUD_PROP], "wc.");
      }
      break;

    case CK_GENERIC:
      fprintf (stderr, "Error, 'wc_properties mie' not implemented for a generic\n");
      fprintf (stderr, "correlated-k calculation\n");
      return -1;

    default:
      fprintf (stderr, "Error, unsupported correlated_k scheme %d\n", Input.ck_scheme);
      return -1;
    }

    fprintf (stderr, " ... reading water cloud optical properties from %s*\n", 
	     Input.filename[FN_WCLOUD_PROP]);
  }
  

  if (strlen(Input.filename[FN_WCLOUD]) > 0 &&
      strlen(Input.filename[FN_WCLOUD_FILES]) > 0)  {
    fprintf (stderr, "Error, it does not make sense to define both wc_file and wc_files.\n");
    fprintf (stderr, "Remove either of them and retry!\n");
    return -1;
  }

  /* ice cloud properties */
  if (Input.ic.properties == ICPROP_MIE) {

    switch (Input.ck_scheme) {
    case CK_KATO:
    case CK_KATO2:
    case CK_KATO2_96:
      if (Input.ic.interpolate)  {
        fprintf (stderr, "* It does not make sense to use \"ic_properties mie\" together with\n");
        fprintf (stderr, "* ic_properties_interpolate for \"correlated_k KATO\" because\n");
	fprintf (stderr, "* in the latter case the cloud properties are pre-calculated\n");
	fprintf (stderr, "* for the required wavelength and interpolation is not useful.\n");
	fprintf (stderr, "* Please remove 'ic_properties_interpolate' from the uvspec input.\n");
	return -1;
      }

      strcpy (Input.filename[FN_ICLOUD_PROP], Input.filename[FN_PATH]);
      strcat (Input.filename[FN_ICLOUD_PROP], "/correlated_k/kato/");
      strcat (Input.filename[FN_ICLOUD_PROP], "ic.");
      break;	    
		    
    case CK_FU:	    
      if (Input.ic.interpolate)  {
        fprintf (stderr, "* It does not make sense to use \"ic_properties mie\" together with\n");
        fprintf (stderr, "* ic_properties_interpolate for \"correlated_k FU\" because\n");
	fprintf (stderr, "* in the latter case the cloud properties are pre-calculated\n");
	fprintf (stderr, "* for the required wavelength and interpolation is not useful.\n");
	fprintf (stderr, "* Please remove 'ic_properties_interpolate' from the uvspec input.\n");
	return -1;
      }

      strcpy (Input.filename[FN_ICLOUD_PROP], Input.filename[FN_PATH]);
      strcat (Input.filename[FN_ICLOUD_PROP], "/correlated_k/fu/");
      strcat (Input.filename[FN_ICLOUD_PROP], "ic.");
      break;
      
    case CK_AVHRR_KRATZ:
      if (Input.ic.interpolate)  {
        fprintf (stderr, "* It does not make sense to use \"ic_properties mie\" together with\n");
        fprintf (stderr, "* ic_properties_interpolate for \"correlated_k AVHRR_KRATZ\" because\n");
	fprintf (stderr, "* in the latter case the cloud properties are pre-calculated\n");
	fprintf (stderr, "* for the required wavelength and interpolation is not useful.\n");
	fprintf (stderr, "* Please remove 'ic_properties_interpolate' from the uvspec input.\n");
	return -1;
      }

      strcpy (Input.filename[FN_ICLOUD_PROP], Input.filename[FN_PATH]);
      strcat (Input.filename[FN_ICLOUD_PROP], "/correlated_k/kratz/");
      strcat (Input.filename[FN_ICLOUD_PROP], "ic.");
      break;

    case CK_NONE:
    case CK_SBDART:
      if (!Input.ic.interpolate)  {
        fprintf (stderr, "* 'ic_properties mie' for a spectral calculation requires that the\n");
        fprintf (stderr, "* cloud optical properties are interpolated to the internal wavelength\n");
	fprintf (stderr, "* grid. Depending on the number of grid points this may require huge\n");
	fprintf (stderr, "* amounts of memory and some additional computational time. To use this\n");
	fprintf (stderr, "* option, please specify explicitely 'ic_properties_interpolate' \n");
	fprintf (stderr, "* in the uvspec input!\n");
	return -1;
      }
      else {
        strcpy (Input.filename[FN_ICLOUD_PROP], Input.filename[FN_PATH]);
        strcat (Input.filename[FN_ICLOUD_PROP], "/ic/mie/");
        strcat (Input.filename[FN_ICLOUD_PROP], "ic.");
      }
      break;

    case CK_GENERIC:
      fprintf (stderr, "Error, 'ic_properties mie' not implemented for a generic\n");
      fprintf (stderr, "correlated-k calculation\n");
      return -1;

    default:
      fprintf (stderr, "Error, unsupported correlated_k scheme %d\n", Input.ck_scheme);
      return -1;
    }

    fprintf (stderr, " ... reading ice cloud optical properties from %s*\n", 
	     Input.filename[FN_ICLOUD_PROP]);
  }
    

  if (strlen(Input.filename[FN_ICLOUD]) > 0 &&
      strlen(Input.filename[FN_ICLOUD_FILES]) > 0)  {
    fprintf (stderr, "Error, it does not make sense to define both ic_file and ic_files.\n");
    fprintf (stderr, "Remove either of them and retry!\n");
    return -1;
  }


  if (Input.rte.mc.momcld==MOMCLD_FILE)
    strcpy (Input.rte.mc.filename[FN_MC_MOMTRNC], Input.rte.mc.filename[FN_MC_MOMCLD]);
  
  if (Input.rte.mc.momcld==MOMCLD_WATER || Input.rte.mc.momcld==MOMCLD_ICE) {

    switch (Input.rte.mc.momcld) {
    case MOMCLD_WATER:
      strcpy (base, "wc.");
      break;
    case MOMCLD_ICE:
      strcpy (base, "ic.");
      break;
    default:
      break;
    }

    switch (Input.ck_scheme) {
    case CK_KATO:
    case CK_KATO2:
    case CK_KATO2_96:
      strcpy (Input.rte.mc.filename[FN_MC_MOMTRNC], Input.filename[FN_PATH]);
      strcat (Input.rte.mc.filename[FN_MC_MOMTRNC], "/correlated_k/kato/");
      strcat (Input.rte.mc.filename[FN_MC_MOMTRNC], base);
      break;

    case CK_FU:
      strcpy (Input.rte.mc.filename[FN_MC_MOMTRNC], Input.filename[FN_PATH]);
      strcat (Input.rte.mc.filename[FN_MC_MOMTRNC], "/correlated_k/fu/");
      strcat (Input.rte.mc.filename[FN_MC_MOMTRNC], base);
      break;
      
    case CK_AVHRR_KRATZ:
      strcpy (Input.rte.mc.filename[FN_MC_MOMTRNC], Input.filename[FN_PATH]);
      strcat (Input.rte.mc.filename[FN_MC_MOMTRNC], "/correlated_k/kratz/");
      strcat (Input.rte.mc.filename[FN_MC_MOMTRNC], base);
      break;

    case CK_NONE:
      fprintf (stderr, "Error, 'mc_momcld_file ice/water' makes only sense together with\n");
      fprintf (stderr, "       'correlated_k'\n");
      return -1;

    default:
      fprintf (stderr, "Error, unsupported correlated_k scheme %d\n", Input.ck_scheme);
      return -1;
    }
  }
	     

  /* aerosol properties */
  if (Input.aer.seasn>0 && Input.aer.vulcan>0 && Input.aer.haze>0 && Input.aer.visibility>0)
    Input.aer.standard = 1;


  /* BRDR stuff */
  brdftype=0;

  /* homogeneous RPV */
  if (Input.rpv.k!=0 || Input.rpv.rho0!=0 || Input.rpv.theta!=0)
    brdftype++;

  /* homogeneous, wavelength-dependent RPV */
  if (strlen(Input.filename[FN_RPV])>0)
    brdftype++;

  /* homogeneous Cox and Munk */
  if (Input.cm.u10 >= 0 || Input.cm.pcl >= 0 || Input.cm.sal >= 0)
    brdftype++;
  
  /* 2D RPV */
  if (strlen(Input.rte.mc.filename[FN_MC_RPV])>0)
    brdftype++;

  if (brdftype > 1)  {
    fprintf (stderr, "Error, it does not make sense to define more than one BRDF type!\n");
    return -1;
  }


  if (Input.rpv.k!=0 || Input.rpv.rho0!=0 || Input.rpv.theta!=0 || 
      strlen(Input.filename[FN_RPV])>0) {
    Input.brdf = BRDF_RPV;

    if (!Input.quiet)
      fprintf (stderr, " ... using RPV parameterization of the BRDF\n");
  }

  if (Input.cm.u10 >= 0 || Input.cm.pcl >= 0 || Input.cm.sal >= 0) {
    Input.brdf = BRDF_CM;

    if (!Input.quiet)
      fprintf (stderr, " ... using Cox and Munk parameterization of the BRDF\n");
  }

  if (strlen(Input.rte.mc.filename[FN_MC_RPV])>0)  {
    Input.brdf = BRDF_RPV2D;

    if (!Input.quiet)
      fprintf (stderr, " ... using 2D RPV parameterization of the BRDF\n");
  }


  /* set default values for windspeed, pigment concentration, and salinity */
  if (Input.brdf == BRDF_CM)  {

    if (Input.cm.u10<0)  {
      Input.cm.u10 = 0;
      if (Input.verbose)
        fprintf (stderr, " ... setting wind speed for Cox and Munk to %f\n", Input.cm.u10);
    }

    if (Input.cm.pcl<0)  {
      Input.cm.pcl = 0.01;
      if (Input.verbose)
        fprintf (stderr, " ... setting pigment concentration for Cox and Munk to %f mg/m3\n", Input.cm.pcl);
    }

    if (Input.cm.sal<0)  {
      Input.cm.sal = 34.3;
      if (Input.verbose)
        fprintf (stderr, " ... setting salinity for Cox and Munk to %f ppm\n", Input.cm.sal);
    }
    
    /* we don't want wind speeds smaller than 1 m/s because the   */
    /* BRDF is expressed as a series of nstr Legendre polynomials */
    /* which causes unwanted structures not only in the BRDF but  */
    /* also in the albedo                                         */
    if (Input.cm.u10<1)  {
    Input.cm.u10 = 1;
      fprintf (stderr, " ... increasing wind speed for Cox and Munk to %.2f m/s\n", Input.cm.u10);
    }
    
    /* we also don't want nstr < 16 for the same reason, even for */
    /* the calculation of fluxes                                  */

    if (Input.rte.nstr<16)  {
      fprintf (stderr, " ... increasing nstr to 16 because of Cox and Munk!\n");
      Input.rte.nstr=16;
    }
  }



  /* Number of phase function moments equals the number of streams  */
  /* per default, but may be changed later in the code;             */
  /* in case of disort2 we increase nmom to at least 1024 to assure */
  /* correct representation of the phase function with only little  */
  /* computational cost                                             */
  Output.atm.nmom = Input.rte.nstr;      
  if (Input.rte.solver == DISORT2 || Input.rte.solver == SSS || Input.rte.solver == SSSI)
    Output.atm.nmom = (Input.rte.nstr>1024 ? Input.rte.nstr : 1024);

  if (Input.rte.solver == SDISORT) {
    if ( Input.rte.nscat != 1 && Input.rte.nscat != 2 ) {
       Input.rte.nscat         = 2;
    }
  }

  Output.atm.molabs = Input.molabs;

  if (Input.rte.nphi <= 0) {
    switch (Input.rte.solver) {
    case SDISORT:
    case SPSDISORT:
    case DISORT:
    case DISORT2:
      Input.rte.maxphi = 3;  /* Minimum number required by disort. */
      Input.rte.nphi   = 1;
      Input.rte.phi    = (float *) calloc (Input.rte.maxphi, sizeof(float));
      Output.print_phi = 0;  /* Do not print radiances if user does not want them */
      break;
    case POLRADTRAN:
      if (Input.rte.pol_aziorder == 0) 
	Input.rte.nphi = 0;
      break;
    }
  }

  if (Input.rte.numu <= 0) {
    Input.rte.maxumu = Input.rte.nstr;
    Input.rte.umu    = (float *) calloc (Input.rte.maxumu, sizeof(float));
  }
  

  /* Output processing */
  if (Input.calibration == OUTCAL_BRIGHTNESS && Input.processing != PROCESS_SUM) {
    fprintf (stderr, "Error, brightness temperatures only implemented for integrated quantities!\n");
    fprintf (stderr, "Please select 'output sum' or 'output integrate'!\n");
    return -1;
  }

  /* Output processing */
  if (Input.calibration == OUTCAL_BRIGHTNESS && Input.source != SRC_THERMAL) {
    fprintf (stderr, "Error, brightness temperatures does only make sense for thermal sources!\n");
    return -1;
  }

  Output.wl.start = Input.wl.start;
  Output.wl.end   = Input.wl.end;

  /* check the wavelength range */
  if (Input.wl.end>500 && (Input.ck_scheme==CK_NONE && strlen(Input.filename[FN_MOLABS])==0))  {
    if (!Input.quiet)  {
      fprintf (stderr, "\n");
      fprintf (stderr, "*** WARNING: You want to do a spectral calculation for wavelengths larger\n");
      fprintf (stderr, "*** than 500 nm. Be warned that from about 500 nm upwards there are highly\n");
      fprintf (stderr, "*** structured molecular absorption bands which are NOT considered in the\n");
      fprintf (stderr, "*** spectral uvspec calculation. While uvspec treats ozone absorption\n");
      fprintf (stderr, "*** correctly, the water vapour absorption bands around 510nm, 545nm,\n");
      fprintf (stderr, "*** 595nm, 655nm, 725nm, 825nm, 935nm, and higher, as well as the\n");
      fprintf (stderr, "*** oxygen absorption at 630nm, 690nm, and 765nm are not implemented.\n");
      fprintf (stderr, "*** Below around 600nm these may usually be neglected; above 600nm it\n");
      fprintf (stderr, "*** starts getting more serious, and above 800nm it does not make any\n");
      fprintf (stderr, "*** sense to do the RTE calculation without considering molecular\n");
      fprintf (stderr, "*** absorption.\n");
      fprintf (stderr, "*** \n");
      fprintf (stderr, "*** To consider molecular absorption other than ozone you have two choices\n");
      fprintf (stderr, "*** with uvspec:\n");
      fprintf (stderr, "***  (1) Do a line-by-line calculation using 'molecular_tau_file' to specify\n");
      fprintf (stderr, "***      the wavelength-dependent absorption profile; to calculate the\n");
      fprintf (stderr, "***      latter, you need something like David Edwards' genln2.\n");
      fprintf (stderr, "***      ATTENTION: line-by-line calculations are very time-consuming!\n");
      fprintf (stderr, "***  (2) Use the correlated-k approximation which is the most accurate\n");
      fprintf (stderr, "***      solution after the line-by-line calculation; use either the\n");
      fprintf (stderr, "***      pre-defined parameterization that come with libRadtran or provide\n");
      fprintf (stderr, "***      your own; both options are selected with 'correlated_k ...'\n");
      fprintf (stderr, "\n");
    }
  }

  /**************************************/
  /* initialize random number generator */
  /**************************************/
  r250_init ((int) time(NULL));

  /********************************/
  /* Do the radiation calculation */
  /********************************/
  
  
  status = uvspec(Input,&Output);

  if (status!=0) {
    fprintf (stderr, "Error %d during execution of uvspec\n", status);
    return status;
  }
  

  /****************************************************************/
  /* Print output ***FIXME*** THIS SHOULD BE USER DEFINED SOMEHOW */
  /****************************************************************/

  print_output(Input, &Output);

  free_output(Input, &Output);
  return status;
}



static int print_output(input_struct Input, output_struct *output) 
{
  int j=0, is=0, iu=0, iv=0, lev=0;

  if (Input.flexstor) {
    printf("flexstor 1 0 suvdama 1 1\n");
    printf("# This flexstor output has not been checked by flxcheck.\n");
    printf("# Hence it may not follow flexstor/suvdama rules. This will\n");
    printf("# be improved soon.\n");
    printf("model_name() uvspec, version %s\n", VERSION);
    if ( fabs(Input.aer.beta-0.0) > 0.0 ) {
      printf("angstrom_alpha()  %f\n", Input.aer.alpha);
      printf("angstrom_beta()  %f\n", Input.aer.beta);
    }
    printf("ozone_column(DU) %f\n", output->ozone_column);
    printf("pressure(mb) %f\n", Input.pressure);
    printf("self_day_of_year() %d\n", Input.day_of_year);
    
    /*
    printf("self_zout(km)");
    for (lev=0;lev<Input.atm.nzout;lev++)
      printf("  %f", Input.atm.zout[lev]);
    printf("\n");
    */

    printf("self_temper(K)");
    for (lev=0;lev<Input.atm.nzout;lev++)
      printf("  %f", output->atm.temper_out[lev]);
    
    printf("\n");
    printf("self_day_of_year() %d\n", Input.day_of_year);
    printf("self_eccentricity_factor() %f\n", output->eccentricity_factor);
    printf("begin table data\n");
    for (lev=0;lev<Input.atm.nzout;lev++) {
      printf("WL(nm) SZA(deg.dec) self_rfldir(mW m-2 nm-2) self_rfldn(mW m-2 nm-2) self_rflup(mW m-2 nm-2)");
      printf(" self_uavgso(mW m-2 nm-2) self_uavgdn(mW m-2 nm-2) self_uavgup(mW m-2 nm-2) ");
    }
    printf("\n");
    for (iv=0;iv<output->wl.nlambda_h;iv++) {
      for (lev=0;lev<Input.atm.nzout;lev++) {
	if ( Input.rte.solver == TWOSTR ) {
	  printf("%7.3f %13.6e %13.6e %13.6e %13.6e ", 
		 output->wl.lambda_h[iv], 
		 output->rfldir[lev][iv], 
		 output->rfldn[lev][iv], 
		 output->flup[lev][iv],
		 output->uavg[lev][iv]);
	}
	else {
	  printf("%7.3f %7.3f %8.5f %13.6e %13.6e %13.6e %13.6e %13.6e %13.6e ",
		 output->wl.lambda_h[iv], 
		 output->sza_h[iv], 
		 output->atm.zout_sur[lev],
		 output->rfldir[lev][iv], 
		 output->rfldn[lev][iv], 
		 output->flup[lev][iv],
		 output->uavgso[lev][iv], 
		 output->uavgdn[lev][iv], 
		 output->uavgup[lev][iv]);
	}
	printf("\n");
      }
    }
    printf("end table data\n");
  }
  else {
    if (Input.header) {
      printf("#aerosol_haze                 : %16d\n",   Input.aer.haze);
      printf("#aerosol_season               : %16d\n",   Input.aer.seasn);
      printf("#aerosol_set_gg               : %16.3f\n", Input.aer.set_gg);
      printf("#aerosol_set_ssa              : %16.3f\n", Input.aer.set_ssa);
      printf("#aerosol_tau_file             : %s\n",     Input.aer.filename[FN_AER_TAU]);
      printf("#aerosol_ssa_file             : %s\n",     Input.aer.filename[FN_AER_SSA]);
      printf("#aerosol_gg_file              : %s\n",     Input.aer.filename[FN_AER_GG]);
      printf("#aerosol_moments_file         : %s\n",     Input.aer.filename[FN_AER_MOM]);
      printf("#aerosol_visibility           : %16.3f\n", Input.aer.visibility);
      printf("#albedo                       : %16.3f\n", Input.atm.albedo);
      printf("#angstrom                     : %16.3f  %16.3f\n", Input.aer.alpha, Input.aer.beta);
      printf("#atmosphere_file              : %s\n", Input.filename[FN_ATMOSPHERE]);
      printf("#day_of_year                  : %16d\n", Input.day_of_year);
      switch (Input.rte.deltam) {
      case 0:
	printf("#deltam                       : off\n");
	break;
      case 1:
	printf("#deltam                       : on\n");
	break;
      default:
	printf("#deltam                       : unknown\n");
	break;
      }

      switch (Input.ozone_spectrum) {
      case O3_BASSPAUR:
	printf("#o3_crs                       : Bass_and_Paur\n");
	break;
      case O3_MOLINA:
	printf("#o3_crs                       : Molina\n");
	break;
      case O3_DAUMONT:
	printf("#o3_crs                       : Daumont\n");
	break;
      default:
	printf("#o3_crs                       : unknown\n");
	break;
      }

      printf("#ozone_column                 : %16.3f\n", Output.ozone_column);

      switch (Input.rte.solver) {
      case SDISORT:
	printf("#rte_solver                   : sdisort\n");
	break;
      case SPSDISORT:
	printf("#rte_solver                   : spsdisort\n");
	break;
      case DISORT:
	printf("#rte_solver                   : disort\n");
	break;
      case DISORT2:
	printf("#rte_solver                   : disort2\n");
	break;
      case MONTECARLO:
	printf("#rte_solver                   : mystic\n");
	break;
      case POLRADTRAN:
	printf("#rte_solver                   : polradtran\n");
	break;
      case SUCORD:
	printf("#rte_solver                   : sucorcsca\n");
	break;
      case TZS: 
	printf("#rte_solver                   : tzs\n");
	break;
      case SSS: 
	printf("#rte_solver                   : sss\n");
	break;
      case SSSI: 
	printf("#rte_solver                   : sss\n");
	break;
      case TWOSTR: 
	printf("#rte_solver                   : twostr\n");
	break;
      default:
	printf("#rte_solver                   : unknown\n");
	break;
      }

      printf("#slit_function_file           : %s\n", 
	     Input.filename[FN_SLITFUNCTION]);
      printf("#solar_file                   : %s\n", 
	     Input.filename[FN_EXTRATERRESTRIAL]);
      printf("#wvn                          : %16.3f  %16.3f\n", 
	     Input.wl.start, Input.wl.end);
      printf("#zout                         : ");
      for (lev=0;lev<Input.atm.nzout;lev++)
	printf(" %16.3f", Input.atm.zout_sur[lev]);
      printf("\n");
    }

    if (Input.rte.solver == POLRADTRAN) {
      for (iv=0;iv<output->wl.nlambda_h;iv++) {
	for (lev=0;lev<Input.atm.nzout;lev++) {
	  printf("%7.3f ", output->wl.lambda_h[iv]);
	  for (is=0;is<Input.rte.pol_nstokes;is++)
	    printf("%13.6e %13.6e ",
		   output->down_flux[lev][is][iv], 
		   output->up_flux[lev][is][iv]);
	    printf("\n");

	  if (output->print_phi && Input.rte.nphi > 0) {
	    printf("                      ");
	    for (j=0; j<Input.rte.nphi; j++)
	      printf(" %13.3f ", Input.rte.phi[j]);
	    printf("\n");
	    for (is=0; is<Input.rte.pol_nstokes; is++) {
	      printf("Stokes vector %c\n", Input.rte.pol_stokes[is]);
	      for (iu=0; iu<Input.rte.numu; iu++) {
/*		printf("%7.4f ", output->mu_values[iu+Input.rte.nstr/2] );*/
		printf("%7.4f ", Input.rte.umu[iu] );
		printf("%13.6e ", 0.0);
		for (j=0; j<Input.rte.nphi; j++) {
		  if (Input.rte.umu[iu] <= 0) {
		     printf(" %13.6e ", output->down_rad[lev][j][iu][is][iv]);}
                  else {
		     printf(" %13.6e ", output->up_rad[lev][j][iu][is][iv]);}
                }
		printf("\n");
	      }
	    }
	  }
	}
      }
    }
    else {
      for (iv=0;iv<output->wl.nlambda_h;iv++) {
	for (lev=0;lev<Input.atm.nzout;lev++) {
	  if (Input.rte.solver == TWOSTR) {
            printf("%9.3f %13.6e %13.6e %13.6e %13.6e\n",  
	    /* printf("%12.6f %13.6e %13.6e %13.6e %13.6e\n", */
		   output->wl.lambda_h[iv], 
		   output->rfldir[lev][iv], 
		   output->rfldn[lev][iv], 
		   output->flup[lev][iv],
		   output->uavg[lev][iv]);
	  }
	  else {
	    printf("%9.3f %13.6e %13.6e %13.6e %13.6e %13.6e %13.6e\n",
	    /* printf("%12.6f %13.6e %13.6e %13.6e %13.6e %13.6e %13.6e\n", */
		   output->wl.lambda_h[iv], 
		   output->rfldir[lev][iv], 
		   output->rfldn[lev][iv], 
		   output->flup[lev][iv],
		   output->uavgso[lev][iv], 
		   output->uavgdn[lev][iv], 
		   output->uavgup[lev][iv]);
	  }

	  if (Input.rte.numu > 0 && !output->print_phi)
            for (iu=0;iu<Input.rte.numu;iu++)
	      printf("%7.4f %13.6e \n", Input.rte.umu[iu], output->u0u[lev][iu][iv]);

	  if (output->print_phi && (Input.rte.solver == DISORT || 
				    Input.rte.solver == DISORT2 ||
				    Input.rte.solver == SDISORT ||
				    Input.rte.solver == SPSDISORT ||
				    Input.rte.solver == SSS ||
				    Input.rte.solver == SSSI ||
				    Input.rte.solver == TZS)
	       && Input.rte.numu> 0) {
	    printf("                      ");
	    for (j=0;j<Input.rte.nphi;j++)
	      printf(" %13.3f ", Input.rte.phi[j]);
	    printf("\n");
	    for (iu=0;iu<Input.rte.numu;iu++) {
	      printf("%7.4f %13.6e ", Input.rte.umu[iu], output->u0u[lev][iu][iv]);
	      for (j=0;j<Input.rte.nphi;j++)
		printf(" %13.6e ", output->uu[lev][j][iu][iv]);
	      printf("\n");
	    }
	  } 
	}
      }    
    }
  }
  fflush(stdout);

  return 0;
}  
  



static void free_output(input_struct Input, output_struct *output) 
{
  if (output->down_flux != NULL)
    ASCII_free_float_3D (output->down_flux, 
	                 Input.atm.nzout, Input.rte.pol_nstokes);
  
  if (output->down_flux_t != NULL)
    ASCII_free_float_3D (output->down_flux_t, 
			 Input.atm.nzout, Input.rte.pol_nstokes);
  
  if (output->up_flux != NULL)
    ASCII_free_float_3D (output->up_flux, 
			 Input.atm.nzout, Input.rte.pol_nstokes);
  
  if (output->up_flux_t != NULL)
    ASCII_free_float_3D (output->up_flux_t, 
			 Input.atm.nzout, Input.rte.pol_nstokes);
  
  if (output->down_rad != NULL)
    ASCII_free_float_5D (output->down_rad, 
			 Input.atm.nzout, Input.rte.nphi, 
			 Input.rte.nstr/2, Input.rte.pol_nstokes);
  
  if (output->down_rad_t != NULL)
    ASCII_free_float_5D (output->down_rad_t, 
			 Input.atm.nzout, Input.rte.nphi,
			 Input.rte.nstr/2, Input.rte.pol_nstokes);
  
  if (output->up_rad != NULL)
    ASCII_free_float_5D (output->up_rad, 
			 Input.atm.nzout, Input.rte.nphi,
			 Input.rte.nstr/2, Input.rte.pol_nstokes);
  
  if (output->up_rad_t != NULL)
    ASCII_free_float_5D (output->up_rad_t, 
			 Input.atm.nzout, Input.rte.nphi,
			 Input.rte.nstr/2, Input.rte.pol_nstokes);
  
  if (output->mc.dtaer != NULL) free(output->mc.dtaer);
  if (output->mc.dtair != NULL) free(output->mc.dtair);
  if (output->mc.dtcld != NULL) free(output->mc.dtcld);
  if (output->mc.dtice != NULL) free(output->mc.dtice);
  if (output->mc.dtmol != NULL) free(output->mc.dtmol);
  if (output->mc.omaer != NULL) free(output->mc.omaer);
  if (output->mc.omcld != NULL) free(output->mc.omcld);
  if (output->mc.omice != NULL) free(output->mc.omice);

  if (output->dtauc != NULL) free(output->dtauc);
  if (output->dtauc_md != NULL) free(output->dtauc_md);
  if (output->sza_h   != NULL) free(output->sza_h);
  if (output->mu_values != NULL) free(output->mu_values);
}


static float yytext2float(char* yytext, int which_token, int max_token) {
  double dnumber=0;
  char token[]=" ";
  char *p=NULL, *s=NULL, *tmpstring=NULL;
  int count=0;
  
  tmpstring = (char *) calloc (strlen(yytext)+1, sizeof (char));
  strcpy (tmpstring, yytext);

  p = strtok(tmpstring,token);
  count = 1;
  while ((p = strtok(NULL,token)) != NULL) {
    count++;
    if (count == which_token)
      s = p;
  }
  
  if ((count != which_token) && (count>max_token))
    fprintf(stderr,"Wrong number of arguments on line %d\n", line_number+1);
  
  dnumber = atof(s);
  return (float) dnumber;
}


static double yytext2double(char* yytext, int which_token, int max_token) {
  double dnumber=0;
  char token[]=" ";
  char *p=NULL, *s=NULL;
  int count=0;
  
  p = strtok(yytext,token);
  count = 1;
  while ((p = strtok(NULL,token)) != NULL) {
    count++;
    if (count == which_token)
      s = p;
  }
  
  if ((count != which_token) && (count>max_token))
    fprintf(stderr,"Wrong number of arguments on line %d\n", line_number+1);
  
  dnumber = atof(s);
  return dnumber;
}


static float* yytext2floats(char* yytext, int nary) {
  int i=0;
  char token[]=" ";
  char *p=NULL;
  
  /* Fill up ary */
  float *ary = (float *) calloc (nary, sizeof(float));
  
  p = strtok(yytext,token);
  i = 0;

  while ((p = strtok(NULL,token)) != NULL)
    ary[i++] = atof(p);
  
  return ary;
}


static int* yytext2integers(char* yytext, int nary) {
  int i=0;
  char token[]=" ";
  char *p=NULL;

  /* Fill up ary */
  int *ary    = (int *) calloc (nary, sizeof(int));

  p = strtok(yytext,token);
  i = 0;

  while ((p = strtok(NULL,token)) != NULL)
    ary[i++] = atoi(p);

  return ary;
}


static int yytext2ntokens(char* yytext) {
  int nary=0;
  char token[]=" ";
  char *p=NULL;

  /* Get number of ary elements */
  nary = 0;
  p = strtok(yytext,token);

  while ((p = strtok(NULL,token)) != NULL) 
    nary++;

  return nary;
}


static int yytext2int(char* yytext, int which_token, int max_token) {
  int inumber=0;
  char token[]=" ";
  char *p=NULL, *s=NULL;
  int count=0;

  p = strtok(yytext,token);
  count = 1;
  while ( (p = strtok(NULL,token)) != NULL) {
    count++;
    if (count == which_token)
      s = p;
  }

  if ((count != which_token) && (count>max_token))
    fprintf(stderr,"Wrong number of arguments on line %d\n", line_number+1);

  inumber = atoi(s);
  
  return inumber;
}


static char* yytext2string(char* yytext, int which_token, int max_token) {
  char token[]=" ";
  char *p=NULL, *s=NULL, *string=NULL, *tmpstring=NULL;
  int count=0;

  tmpstring = (char *) calloc (strlen(yytext)+1, sizeof (char));
  strcpy (tmpstring, yytext);

  p = strtok(tmpstring,token);
  count = 1;
  while ( (p = strtok(NULL,token)) != NULL) {
    count++;
    if (count == which_token)
      s = p;
  }

  if ((count != which_token) && (count>max_token))
    fprintf(stderr,"Wrong number of arguments on line %d\n", line_number+1);

  string = (char *) calloc (strlen(s)+1, sizeof (char));
  strcpy (string, s);
  return string;
}


/* like yytext2string, but string is copied to a pre-allocated string dest */
static int yytextCstring(char *dest, char* yytext, int which_token, int max_token) {
  char token[]=" ";
  char *p=NULL, *s=NULL, *tmpstring=NULL;
  int count=0;

  tmpstring = (char *) calloc (strlen(yytext)+1, sizeof (char));
  strcpy (tmpstring, yytext);

  p = strtok(tmpstring,token);
  count = 1;
  while ( (p = strtok(NULL,token)) != NULL) {
    count++;
    if (count == which_token)
      s = p;
  }

  if ((count != which_token) && (count>max_token))
    fprintf(stderr,"Wrong number of arguments on line %d\n", line_number+1);

  strcpy (dest, s);
  return 0;
}
